import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LibCxxStdFunctionRecognizerTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @add_test_categories(["libc++"])
    def test_frame_recognizer(self):
        """Test that std::function all implementation details are hidden in SBFrame"""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        self.assertIn("foo", thread.GetFrameAtIndex(0).GetFunctionName())
        # Skip all hidden frames
        frame_id = 1
        while (
            frame_id < thread.GetNumFrames()
            and thread.GetFrameAtIndex(frame_id).IsHidden()
        ):
            frame_id = frame_id + 1
        # Expect `std::function<...>::operator()` to be the direct parent of `foo`
        self.assertIn(
            "::operator()", thread.GetFrameAtIndex(frame_id).GetFunctionName()
        )
        # And right above that, there should be the `main` frame
        self.assertIn("main", thread.GetFrameAtIndex(frame_id + 1).GetFunctionName())

    @add_test_categories(["libc++"])
    def test_backtrace(self):
        """Test that std::function implementation details are hidden in bt"""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        # Filtered.
        self.expect(
            "thread backtrace",
            ordered=True,
            substrs=["frame", "foo", "frame", "main"],
        )
        self.expect(
            "thread backtrace", matching=False, patterns=["frame.*std::__.*::__function"]
        )
        # Unfiltered.
        self.expect(
            "bt -u",
            ordered=True,
            patterns=["frame.*foo", "frame.*std::__[^:]*::__function", "frame.*main"],
        )
        self.expect(
            "thread backtrace -u",
            ordered=True,
            patterns=["frame.*foo", "frame.*std::__[^:]*::__function", "frame.*main"],
        )
        self.expect(
            "thread backtrace --unfiltered",
            ordered=True,
            patterns=["frame.*foo", "frame.*std::__[^:]*::__function", "frame.*main"],
        )

    @add_test_categories(["libc++"])
    def test_up_down(self):
        """Test that std::function implementation details are skipped"""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        frame = thread.selected_frame
        # up
        self.assertIn("foo", frame.GetFunctionName())
        start_idx = frame.GetFrameID()
        i = 0
        while i < thread.GetNumFrames():
            self.expect("up")
            frame = thread.selected_frame
            if frame.GetFunctionName() == "main":
                break
        end_idx = frame.GetFrameID()
        self.assertLess(i, end_idx - start_idx, "skipped frames")

        # Back down again.
        start_idx = frame.GetFrameID()
        for i in range(1, thread.GetNumFrames()):
            self.expect("down")
            frame = thread.selected_frame
            if "foo" in frame.GetFunctionName():
                break
        end_idx = frame.GetFrameID()
        self.assertLess(i, start_idx - end_idx, "skipped frames")

    @add_test_categories(["libc++"])
    def test_api(self):
        """Test that std::function implementation details are skipped"""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        num_hidden = 0
        for frame in thread.frames:
            if frame.IsHidden():
                num_hidden += 1

        self.assertGreater(num_hidden, 0)
        self.assertLess(num_hidden, thread.GetNumFrames())



"""
Tests calling a function by basename
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CallCPPFunctionTestCase(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.line = line_number("main.cpp", "// breakpoint")

    def test_with_run_command(self):
        """Test calling a function by basename"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// breakpoint", lldb.SBFileSpec("main.cpp")
        )

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.expect_expr("a_function_to_call()", result_type="int", result_value="0")



# RUN: %PYTHON %s | FileCheck %s
from functools import partialmethod

from mlir.ir import *
import mlir.dialects.arith as arith
import mlir.dialects.func as func
from array import array


def run(f):
    print("\nTEST:", f.__name__)
    f()


# CHECK-LABEL: TEST: testConstantOp
@run
def testConstantOps():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            arith.ConstantOp(value=42.42, result=F32Type.get())
        # CHECK:         %cst = arith.constant 4.242000e+01 : f32
        print(module)


# CHECK-LABEL: TEST: testFastMathFlags
@run
def testFastMathFlags():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            a = arith.ConstantOp(value=42.42, result=F32Type.get())
            r = arith.AddFOp(
                a, a, fastmath=arith.FastMathFlags.nnan | arith.FastMathFlags.ninf
            )
            # CHECK: %0 = arith.addf %cst, %cst fastmath<nnan,ninf> : f32
            print(r)


# CHECK-LABEL: TEST: testArithValue
@run
def testArithValue():
    def _binary_op(lhs, rhs, op: str) -> "ArithValue":
        op = op.capitalize()
        if isinstance(lhs.type, FloatType) and isinstance(rhs.type, FloatType):
            op += "F"
        elif isinstance(lhs.type, (IntegerType, IndexType)) and isinstance(
            lhs.type, (IntegerType, IndexType)
        ):
            op += "I"
        else:
            raise NotImplementedError(f"Unsupported '{op}' operands: {lhs}, {rhs}")

        op = getattr(arith, f"{op}Op")
        return op(lhs, rhs).result

    @register_value_caster(F16Type.static_typeid)
    @register_value_caster(F32Type.static_typeid)
    @register_value_caster(F64Type.static_typeid)
    @register_value_caster(IntegerType.static_typeid)
    class ArithValue(Value):
        def __init__(self, v):
            super().__init__(v)

        __add__ = partialmethod(_binary_op, op="add")
        __sub__ = partialmethod(_binary_op, op="sub")
        __mul__ = partialmethod(_binary_op, op="mul")

        def __str__(self):
            return super().__str__().replace(Value.__name__, ArithValue.__name__)

    with Context() as ctx, Location.unknown():
        module = Module.create()
        f16_t = F16Type.get()
        f32_t = F32Type.get()
        f64_t = F64Type.get()

        with InsertionPoint(module.body):
            a = arith.constant(f16_t, 42.42)
            # CHECK: ArithValue(%cst = arith.constant 4.240
            print(a)

            b = a + a
            # CHECK: ArithValue(%0 = arith.addf %cst, %cst : f16)
            print(b)

            a = arith.constant(f32_t, 42.42)
            b = a - a
            # CHECK: ArithValue(%1 = arith.subf %cst_0, %cst_0 : f32)
            print(b)

            a = arith.constant(f64_t, 42.42)
            b = a * a
            # CHECK: ArithValue(%2 = arith.mulf %cst_1, %cst_1 : f64)
            print(b)


# CHECK-LABEL: TEST: testArrayConstantConstruction
@run
def testArrayConstantConstruction():
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            i32_array = array("i", [1, 2, 3, 4])
            i32 = IntegerType.get_signless(32)
            vec_i32 = VectorType.get([2, 2], i32)
            arith.constant(vec_i32, i32_array)
            arith.ConstantOp(vec_i32, DenseIntElementsAttr.get(i32_array, type=vec_i32))

            # "q" is the equivalent of `long long` in C and requires at least
            # 64 bit width integers on both Linux and Windows.
            i64_array = array("q", [5, 6, 7, 8])
            i64 = IntegerType.get_signless(64)
            vec_i64 = VectorType.get([1, 4], i64)
            arith.constant(vec_i64, i64_array)
            arith.ConstantOp(vec_i64, DenseIntElementsAttr.get(i64_array, type=vec_i64))

            f32_array = array("f", [1.0, 2.0, 3.0, 4.0])
            f32 = F32Type.get()
            vec_f32 = VectorType.get([4, 1], f32)
            arith.constant(vec_f32, f32_array)
            arith.ConstantOp(vec_f32, DenseFPElementsAttr.get(f32_array, type=vec_f32))

            f64_array = array("d", [1.0, 2.0, 3.0, 4.0])
            f64 = F64Type.get()
            vec_f64 = VectorType.get([2, 1, 2], f64)
            arith.constant(vec_f64, f64_array)
            arith.ConstantOp(vec_f64, DenseFPElementsAttr.get(f64_array, type=vec_f64))

        # CHECK-COUNT-2: arith.constant dense<[{{\[}}1, 2], [3, 4]]> : vector<2x2xi32>
        # CHECK-COUNT-2: arith.constant dense<[{{\[}}5, 6, 7, 8]]> : vector<1x4xi64>
        # CHECK-COUNT-2: arith.constant dense<[{{\[}}1.000000e+00], [2.000000e+00], [3.000000e+00], [4.000000e+00]]> : vector<4x1xf32>
        # CHECK-COUNT-2: arith.constant dense<[{{\[}}[1.000000e+00, 2.000000e+00]], [{{\[}}3.000000e+00, 4.000000e+00]]]> : vector<2x1x2xf64>
        print(module)



"""
Base class for gdb-remote test cases.
"""

import errno
import os
import os.path
import random
import re
import select
import socket
import subprocess
import sys
import tempfile
import time
from lldbsuite.test import configuration
from lldbsuite.test.lldbtest import *
from lldbsuite.support import seven
from lldbgdbserverutils import *
import logging


class _ConnectionRefused(IOError):
    pass


class GdbRemoteTestCaseFactory(type):
    def __new__(cls, name, bases, attrs):
        newattrs = {}
        for attrname, attrvalue in attrs.items():
            if not attrname.startswith("test"):
                newattrs[attrname] = attrvalue
                continue

            # If any debug server categories were explicitly tagged, assume
            # that list to be authoritative. If none were specified, try
            # all of them.
            all_categories = set(["debugserver", "llgs"])
            categories = set(getattr(attrvalue, "categories", [])) & all_categories
            if not categories:
                categories = all_categories

            for cat in categories:

                @decorators.add_test_categories([cat])
                @wraps(attrvalue)
                def test_method(self, attrvalue=attrvalue):
                    return attrvalue(self)

                method_name = attrname + "_" + cat
                test_method.__name__ = method_name
                test_method.debug_server = cat
                newattrs[method_name] = test_method

        return super(GdbRemoteTestCaseFactory, cls).__new__(cls, name, bases, newattrs)


class GdbRemoteTestCaseBase(Base, metaclass=GdbRemoteTestCaseFactory):
    # Default time out in seconds. The timeout is increased tenfold under Asan.
    DEFAULT_TIMEOUT = 20 * (10 if ("ASAN_OPTIONS" in os.environ) else 1)
    # Default sleep time in seconds. The sleep time is doubled under Asan.
    DEFAULT_SLEEP = 5 * (2 if ("ASAN_OPTIONS" in os.environ) else 1)

    _GDBREMOTE_KILL_PACKET = b"$k#6b"

    # Start the inferior separately, attach to the inferior on the stub
    # command line.
    _STARTUP_ATTACH = "attach"
    # Start the inferior separately, start the stub without attaching, allow
    # the test to attach to the inferior however it wants (e.g. $vAttach;pid).
    _STARTUP_ATTACH_MANUALLY = "attach_manually"
    # Start the stub, and launch the inferior with an $A packet via the
    # initial packet stream.
    _STARTUP_LAUNCH = "launch"

    # GDB Signal numbers that are not target-specific used for common
    # exceptions
    TARGET_EXC_BAD_ACCESS = 0x91
    TARGET_EXC_BAD_INSTRUCTION = 0x92
    TARGET_EXC_ARITHMETIC = 0x93
    TARGET_EXC_EMULATION = 0x94
    TARGET_EXC_SOFTWARE = 0x95
    TARGET_EXC_BREAKPOINT = 0x96

    _verbose_log_handler = None
    _log_formatter = logging.Formatter(fmt="%(asctime)-15s %(levelname)-8s %(message)s")

    def setUpBaseLogging(self):
        self.logger = logging.getLogger(__name__)

        if len(self.logger.handlers) > 0:
            return  # We have set up this handler already

        self.logger.propagate = False
        self.logger.setLevel(logging.DEBUG)

        # log all warnings to stderr
        handler = logging.StreamHandler()
        handler.setLevel(logging.WARNING)
        handler.setFormatter(self._log_formatter)
        self.logger.addHandler(handler)

    def isVerboseLoggingRequested(self):
        # We will report our detailed logs if the user requested that the "gdb-remote" channel is
        # logged.
        return any(("gdb-remote" in channel) for channel in lldbtest_config.channels)

    def getDebugServer(self):
        method = getattr(self, self.testMethodName)
        return getattr(method, "debug_server", None)

    def setUp(self):
        super(GdbRemoteTestCaseBase, self).setUp()

        self.setUpBaseLogging()
        self.debug_monitor_extra_args = []

        if self.isVerboseLoggingRequested():
            # If requested, full logs go to a log file
            self._verbose_log_handler = logging.FileHandler(
                self.getLogBasenameForCurrentTest() + "-host.log"
            )
            self._verbose_log_handler.setFormatter(self._log_formatter)
            self._verbose_log_handler.setLevel(logging.DEBUG)
            self.logger.addHandler(self._verbose_log_handler)

        self.test_sequence = GdbRemoteTestSequence(self.logger)
        self.set_inferior_startup_launch()
        self.port = self.get_next_port()
        self.stub_sends_two_stop_notifications_on_kill = False
        if configuration.lldb_platform_url:
            if configuration.lldb_platform_url.startswith("unix-"):
                url_pattern = r"(.+)://\[?(.+?)\]?/.*"
            else:
                url_pattern = r"(.+)://(.+):\d+"
            scheme, host = re.match(
                url_pattern, configuration.lldb_platform_url
            ).groups()
            if (
                configuration.lldb_platform_name == "remote-android"
                and host != "localhost"
            ):
                self.stub_device = host
                self.stub_hostname = "localhost"
            else:
                self.stub_device = None
                self.stub_hostname = host
        else:
            self.stub_hostname = "localhost"

        debug_server = self.getDebugServer()
        if debug_server == "debugserver":
            self._init_debugserver_test()
        else:
            self._init_llgs_test()

    def tearDown(self):
        self.logger.removeHandler(self._verbose_log_handler)
        self._verbose_log_handler = None
        TestBase.tearDown(self)

    def getLocalServerLogFile(self):
        return self.getLogBasenameForCurrentTest() + "-server.log"

    def setUpServerLogging(self, is_llgs):
        if len(lldbtest_config.channels) == 0:
            return  # No logging requested

        if lldb.remote_platform:
            log_file = lldbutil.join_remote_paths(
                lldb.remote_platform.GetWorkingDirectory(), "server.log"
            )
        else:
            log_file = self.getLocalServerLogFile()

        if is_llgs:
            self.debug_monitor_extra_args.append("--log-file=" + log_file)
            self.debug_monitor_extra_args.append(
                "--log-channels={}".format(":".join(lldbtest_config.channels))
            )
        else:
            self.debug_monitor_extra_args = [
                "--log-file=" + log_file,
                "--log-flags=0x800000",
            ]

    def get_next_port(self):
        if available_ports := self.getPlatformAvailablePorts():
            return random.choice(available_ports)
        return 12000 + random.randint(0, 7999)

    def reset_test_sequence(self):
        self.test_sequence = GdbRemoteTestSequence(self.logger)

    def _init_llgs_test(self):
        reverse_connect = True
        if lldb.remote_platform:
            # Reverse connections may be tricky due to firewalls/NATs.
            reverse_connect = False

            # FIXME: This is extremely linux-oriented

            # Grab the ppid from /proc/[shell pid]/stat
            err, retcode, shell_stat = self.run_platform_command("cat /proc/$$/stat")
            self.assertTrue(
                err.Success() and retcode == 0,
                "Failed to read file /proc/$$/stat: %s, retcode: %d"
                % (err.GetCString(), retcode),
            )

            # [pid] ([executable]) [state] [*ppid*]
            pid = re.match(r"^\d+ \(.+\) . (\d+)", shell_stat).group(1)
            err, retcode, ls_output = self.run_platform_command(
                "ls -l /proc/%s/exe" % pid
            )
            self.assertTrue(
                err.Success() and retcode == 0,
                "Failed to read file /proc/%s/exe: %s, retcode: %d"
                % (pid, err.GetCString(), retcode),
            )
            exe = ls_output.split()[-1]

            # If the binary has been deleted, the link name has " (deleted)" appended.
            # Remove if it's there.
            self.debug_monitor_exe = re.sub(r" \(deleted\)$", "", exe)
        else:
            self.debug_monitor_exe = get_lldb_server_exe()

        self.debug_monitor_extra_args = ["gdbserver"]
        self.setUpServerLogging(is_llgs=True)

        self.reverse_connect = reverse_connect

    def _init_debugserver_test(self):
        self.debug_monitor_exe = get_debugserver_exe()
        self.setUpServerLogging(is_llgs=False)
        self.reverse_connect = True

        # The debugserver stub has a race on handling the 'k' command, so it sends an X09 right away, then sends the real X notification
        # when the process truly dies.
        self.stub_sends_two_stop_notifications_on_kill = True

    def forward_adb_port(self, source, target, direction, device):
        adb = ["adb"] + (["-s", device] if device else []) + [direction]

        def remove_port_forward():
            subprocess.call(adb + ["--remove", "tcp:%d" % source])

        subprocess.call(adb + ["tcp:%d" % source, "tcp:%d" % target])
        self.addTearDownHook(remove_port_forward)

    def _verify_socket(self, sock):
        # Normally, when the remote stub is not ready, we will get ECONNREFUSED during the
        # connect() attempt. However, due to the way how port forwarding can work, on some targets
        # the connect() will always be successful, but the connection will be immediately dropped
        # if we could not connect on the remote side. This function tries to detect this
        # situation, and report it as "connection refused" so that the upper layers attempt the
        # connection again.
        can_read, _, _ = select.select([sock], [], [], 0.1)
        if sock not in can_read:
            return  # Data is not available, but the connection is alive.
        if len(sock.recv(1, socket.MSG_PEEK)) == 0:
            raise _ConnectionRefused()  # Got EOF, connection dropped.

    def create_socket(self):
        try:
            sock = socket.socket(family=socket.AF_INET)
        except OSError as e:
            if e.errno != errno.EAFNOSUPPORT:
                raise
            sock = socket.socket(family=socket.AF_INET6)

        logger = self.logger

        triple = self.dbg.GetSelectedPlatform().GetTriple()
        if re.match(".*-.*-.*-android", triple):
            self.forward_adb_port(self.port, self.port, "forward", self.stub_device)

        logger.info(
            "Connecting to debug monitor on %s:%d", self.stub_hostname, self.port
        )
        connect_info = (self.stub_hostname, self.port)
        try:
            sock.connect(connect_info)
        except socket.error as serr:
            if serr.errno == errno.ECONNREFUSED:
                raise _ConnectionRefused()
            raise serr

        def shutdown_socket():
            if sock:
                try:
                    # send the kill packet so lldb-server shuts down gracefully
                    sock.sendall(GdbRemoteTestCaseBase._GDBREMOTE_KILL_PACKET)
                except:
                    logger.warning(
                        "failed to send kill packet to debug monitor: {}; ignoring".format(
                            sys.exc_info()[0]
                        )
                    )

                try:
                    sock.close()
                except:
                    logger.warning(
                        "failed to close socket to debug monitor: {}; ignoring".format(
                            sys.exc_info()[0]
                        )
                    )

        self.addTearDownHook(shutdown_socket)

        self._verify_socket(sock)

        return sock

    def set_inferior_startup_launch(self):
        self._inferior_startup = self._STARTUP_LAUNCH

    def set_inferior_startup_attach(self):
        self._inferior_startup = self._STARTUP_ATTACH

    def set_inferior_startup_attach_manually(self):
        self._inferior_startup = self._STARTUP_ATTACH_MANUALLY

    def get_debug_monitor_command_line_args(self, attach_pid=None):
        commandline_args = self.debug_monitor_extra_args
        if attach_pid:
            commandline_args += ["--attach=%d" % attach_pid]
        if self.reverse_connect:
            commandline_args += ["--reverse-connect", self.connect_address]
        else:
            if lldb.remote_platform:
                commandline_args += ["*:{}".format(self.port)]
            else:
                commandline_args += ["localhost:{}".format(self.port)]

        return commandline_args

    def get_target_byte_order(self):
        inferior_exe_path = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(inferior_exe_path)
        return target.GetByteOrder()

    def launch_debug_monitor(self, attach_pid=None, logfile=None):
        if self.reverse_connect:
            family, type, proto, _, addr = socket.getaddrinfo(
                "localhost", 0, proto=socket.IPPROTO_TCP
            )[0]
            sock = socket.socket(family, type, proto)
            sock.settimeout(self.DEFAULT_TIMEOUT)

            sock.bind(addr)
            sock.listen(1)
            addr = sock.getsockname()
            self.connect_address = "[{}]:{}".format(*addr)

        # Create the command line.
        commandline_args = self.get_debug_monitor_command_line_args(
            attach_pid=attach_pid
        )

        # Start the server.
        server = self.spawnSubprocess(
            self.debug_monitor_exe, commandline_args, install_remote=False
        )
        self.assertIsNotNone(server)

        if self.reverse_connect:
            self.sock = sock.accept()[0]
            self.sock.settimeout(self.DEFAULT_TIMEOUT)

        return server

    def connect_to_debug_monitor(self, attach_pid=None):
        if self.reverse_connect:
            # Create the stub.
            server = self.launch_debug_monitor(attach_pid=attach_pid)
            self.assertIsNotNone(server)

            # Schedule debug monitor to be shut down during teardown.
            logger = self.logger

            self._server = Server(self.sock, server)
            return server

        # We're using a random port algorithm to try not to collide with other ports,
        # and retry a max # times.
        attempts = 0
        MAX_ATTEMPTS = 10
        attempt_wait = 3

        while attempts < MAX_ATTEMPTS:
            server = self.launch_debug_monitor(attach_pid=attach_pid)

            # Schedule debug monitor to be shut down during teardown.
            logger = self.logger

            connect_attempts = 0
            MAX_CONNECT_ATTEMPTS = 10

            while connect_attempts < MAX_CONNECT_ATTEMPTS:
                # Create a socket to talk to the server
                try:
                    logger.info("Connect attempt %d", connect_attempts + 1)
                    self.sock = self.create_socket()
                    self._server = Server(self.sock, server)
                    return server
                except _ConnectionRefused as serr:
                    # Ignore, and try again.
                    pass
                time.sleep(0.5)
                connect_attempts += 1

            # We should close the server here to be safe.
            server.terminate()

            # Increment attempts.
            print(
                "connect to debug monitor on port %d failed, attempt #%d of %d"
                % (self.port, attempts + 1, MAX_ATTEMPTS)
            )
            attempts += 1

            # And wait a random length of time before next attempt, to avoid
            # collisions.
            time.sleep(attempt_wait)
            attempt_wait *= 1.2

            # Now grab a new port number.
            self.port = self.get_next_port()

        raise Exception(
            "failed to create a socket to the launched debug monitor after %d tries"
            % attempts
        )

    def launch_process_for_attach(
        self, inferior_args=None, sleep_seconds=3, exe_path=None
    ):
        # We're going to start a child process that the debug monitor stub can later attach to.
        # This process needs to be started so that it just hangs around for a while.  We'll
        # have it sleep.
        if not exe_path:
            exe_path = self.getBuildArtifact("a.out")

        # This file will be created once the inferior has enabled attaching.
        sync_file_path = lldbutil.append_to_process_working_directory(
            self, "process_ready"
        )
        args = [f"syncfile:{sync_file_path}"]
        if inferior_args:
            args.extend(inferior_args)
        if sleep_seconds:
            args.append("sleep:%d" % sleep_seconds)

        inferior = self.spawnSubprocess(exe_path, args)
        lldbutil.wait_for_file_on_target(self, sync_file_path)

        return inferior

    def prep_debug_monitor_and_inferior(
        self,
        inferior_args=None,
        inferior_sleep_seconds=3,
        inferior_exe_path=None,
        inferior_env=None,
    ):
        """Prep the debug monitor, the inferior, and the expected packet stream.

        Handle the separate cases of using the debug monitor in attach-to-inferior mode
        and in launch-inferior mode.

        For attach-to-inferior mode, the inferior process is first started, then
        the debug monitor is started in attach to pid mode (using --attach on the
        stub command line), and the no-ack-mode setup is appended to the packet
        stream.  The packet stream is not yet executed, ready to have more expected
        packet entries added to it.

        For launch-inferior mode, the stub is first started, then no ack mode is
        setup on the expected packet stream, then the verified launch packets are added
        to the expected socket stream.  The packet stream is not yet executed, ready
        to have more expected packet entries added to it.

        The return value is:
        {inferior:<inferior>, server:<server>}
        """
        inferior = None
        attach_pid = None

        if (
            self._inferior_startup == self._STARTUP_ATTACH
            or self._inferior_startup == self._STARTUP_ATTACH_MANUALLY
        ):
            # Launch the process that we'll use as the inferior.
            inferior = self.launch_process_for_attach(
                inferior_args=inferior_args,
                sleep_seconds=inferior_sleep_seconds,
                exe_path=inferior_exe_path,
            )
            self.assertIsNotNone(inferior)
            self.assertTrue(inferior.pid > 0)
            if self._inferior_startup == self._STARTUP_ATTACH:
                # In this case, we want the stub to attach via the command
                # line, so set the command line attach pid here.
                attach_pid = inferior.pid

        if self._inferior_startup == self._STARTUP_LAUNCH:
            # Build launch args
            if not inferior_exe_path:
                inferior_exe_path = self.getBuildArtifact("a.out")

            if lldb.remote_platform:
                remote_path = lldbutil.append_to_process_working_directory(
                    self, os.path.basename(inferior_exe_path)
                )
                remote_file_spec = lldb.SBFileSpec(remote_path, False)
                err = lldb.remote_platform.Install(
                    lldb.SBFileSpec(inferior_exe_path, True), remote_file_spec
                )
                if err.Fail():
                    raise Exception(
                        "remote_platform.Install('%s', '%s') failed: %s"
                        % (inferior_exe_path, remote_path, err)
                    )
                inferior_exe_path = remote_path

            launch_args = [inferior_exe_path]
            if inferior_args:
                launch_args.extend(inferior_args)

        # Launch the debug monitor stub, attaching to the inferior.
        server = self.connect_to_debug_monitor(attach_pid=attach_pid)
        self.assertIsNotNone(server)

        self.do_handshake()

        # Build the expected protocol stream
        if inferior_env:
            for name, value in inferior_env.items():
                self.add_set_environment_packets(name, value)
        if self._inferior_startup == self._STARTUP_LAUNCH:
            self.add_verified_launch_packets(launch_args)

        return {"inferior": inferior, "server": server}

    def do_handshake(self):
        server = self._server
        server.send_ack()
        server.send_packet(b"QStartNoAckMode")
        self.assertEqual(server.get_normal_packet(), b"+")
        self.assertEqual(server.get_normal_packet(), b"OK")
        server.send_ack()

    def add_verified_launch_packets(self, launch_args):
        self.test_sequence.add_log_lines(
            [
                "read packet: %s" % build_gdbremote_A_packet(launch_args),
                "send packet: $OK#00",
                "read packet: $qLaunchSuccess#a5",
                "send packet: $OK#00",
            ],
            True,
        )

    def add_thread_suffix_request_packets(self):
        self.test_sequence.add_log_lines(
            [
                "read packet: $QThreadSuffixSupported#e4",
                "send packet: $OK#00",
            ],
            True,
        )

    def add_process_info_collection_packets(self):
        self.test_sequence.add_log_lines(
            [
                "read packet: $qProcessInfo#dc",
                {
                    "direction": "send",
                    "regex": r"^\$(.+)#[0-9a-fA-F]{2}$",
                    "capture": {1: "process_info_raw"},
                },
            ],
            True,
        )

    def add_set_environment_packets(self, name, value):
        self.test_sequence.add_log_lines(
            [
                "read packet: $QEnvironment:" + name + "=" + value + "#00",
                "send packet: $OK#00",
            ],
            True,
        )

    _KNOWN_PROCESS_INFO_KEYS = [
        "pid",
        "parent-pid",
        "real-uid",
        "real-gid",
        "effective-uid",
        "effective-gid",
        "cputype",
        "cpusubtype",
        "ostype",
        "triple",
        "vendor",
        "endian",
        "elf_abi",
        "ptrsize",
    ]

    def parse_process_info_response(self, context):
        # Ensure we have a process info response.
        self.assertIsNotNone(context)
        process_info_raw = context.get("process_info_raw")
        self.assertIsNotNone(process_info_raw)

        # Pull out key:value; pairs.
        process_info_dict = {
            match.group(1): match.group(2)
            for match in re.finditer(r"([^:]+):([^;]+);", process_info_raw)
        }

        # Validate keys are known.
        for key, val in list(process_info_dict.items()):
            self.assertTrue(key in self._KNOWN_PROCESS_INFO_KEYS)
            self.assertIsNotNone(val)

        return process_info_dict

    def add_register_info_collection_packets(self):
        self.test_sequence.add_log_lines(
            [
                {
                    "type": "multi_response",
                    "query": "qRegisterInfo",
                    "append_iteration_suffix": True,
                    "end_regex": re.compile(r"^\$(E\d+)?#[0-9a-fA-F]{2}$"),
                    "save_key": "reg_info_responses",
                }
            ],
            True,
        )

    def parse_register_info_packets(self, context):
        """Return an array of register info dictionaries, one per register info."""
        reg_info_responses = context.get("reg_info_responses")
        self.assertIsNotNone(reg_info_responses)

        # Parse register infos.
        return [
            parse_reg_info_response(reg_info_response)
            for reg_info_response in reg_info_responses
        ]

    def expect_gdbremote_sequence(self):
        return expect_lldb_gdbserver_replay(
            self,
            self._server,
            self.test_sequence,
            self.DEFAULT_TIMEOUT * len(self.test_sequence),
            self.logger,
        )

    _KNOWN_REGINFO_KEYS = [
        "name",
        "alt-name",
        "bitsize",
        "offset",
        "encoding",
        "format",
        "set",
        "gcc",
        "ehframe",
        "dwarf",
        "generic",
        "container-regs",
        "invalidate-regs",
        "dynamic_size_dwarf_expr_bytes",
        "dynamic_size_dwarf_len",
    ]

    def assert_valid_reg_info(self, reg_info):
        # Assert we know about all the reginfo keys parsed.
        for key in reg_info:
            self.assertTrue(key in self._KNOWN_REGINFO_KEYS)

        # Check the bare-minimum expected set of register info keys.
        self.assertTrue("name" in reg_info)
        self.assertTrue("bitsize" in reg_info)

        if not (self.getArchitecture() == "aarch64" or self.isRISCV()):
            self.assertTrue("offset" in reg_info)

        self.assertTrue("encoding" in reg_info)
        self.assertTrue("format" in reg_info)

    def find_pc_reg_info(self, reg_infos):
        lldb_reg_index = 0
        for reg_info in reg_infos:
            if ("generic" in reg_info) and (reg_info["generic"] == "pc"):
                return (lldb_reg_index, reg_info)
            lldb_reg_index += 1

        return (None, None)

    def add_lldb_register_index(self, reg_infos):
        """Add a "lldb_register_index" key containing the 0-baed index of each reg_infos entry.

        We'll use this when we want to call packets like P/p with a register index but do so
        on only a subset of the full register info set.
        """
        self.assertIsNotNone(reg_infos)

        reg_index = 0
        for reg_info in reg_infos:
            reg_info["lldb_register_index"] = reg_index
            reg_index += 1

    def add_query_memory_region_packets(self, address):
        self.test_sequence.add_log_lines(
            [
                "read packet: $qMemoryRegionInfo:{0:x}#00".format(address),
                {
                    "direction": "send",
                    "regex": r"^\$(.+)#[0-9a-fA-F]{2}$",
                    "capture": {1: "memory_region_response"},
                },
            ],
            True,
        )

    def parse_key_val_dict(self, key_val_text, allow_dupes=True):
        self.assertIsNotNone(key_val_text)
        kv_dict = {}
        for match in re.finditer(r";?([^:]+):([^;]+)", key_val_text):
            key = match.group(1)
            val = match.group(2)
            if key in kv_dict:
                if allow_dupes:
                    if isinstance(kv_dict[key], list):
                        kv_dict[key].append(val)
                    else:
                        # Promote to list
                        kv_dict[key] = [kv_dict[key], val]
                else:
                    self.fail(
                        "key '{}' already present when attempting to add value '{}' (text='{}', dict={})".format(
                            key, val, key_val_text, kv_dict
                        )
                    )
            else:
                kv_dict[key] = val
        return kv_dict

    def parse_memory_region_packet(self, context):
        # Ensure we have a context.
        self.assertIsNotNone(context.get("memory_region_response"))

        # Pull out key:value; pairs.
        mem_region_dict = self.parse_key_val_dict(context.get("memory_region_response"))

        # Validate keys are known.
        for key, val in list(mem_region_dict.items()):
            self.assertIn(
                key,
                [
                    "start",
                    "size",
                    "permissions",
                    "flags",
                    "name",
                    "error",
                    "dirty-pages",
                    "type",
                ],
            )
            self.assertIsNotNone(val)

        mem_region_dict["name"] = seven.unhexlify(mem_region_dict.get("name", ""))
        # Return the dictionary of key-value pairs for the memory region.
        return mem_region_dict

    def assert_address_within_memory_region(self, test_address, mem_region_dict):
        self.assertIsNotNone(mem_region_dict)
        self.assertTrue("start" in mem_region_dict)
        self.assertTrue("size" in mem_region_dict)

        range_start = int(mem_region_dict["start"], 16)
        range_size = int(mem_region_dict["size"], 16)
        range_end = range_start + range_size

        if test_address < range_start:
            self.fail(
                "address 0x{0:x} comes before range 0x{1:x} - 0x{2:x} (size 0x{3:x})".format(
                    test_address, range_start, range_end, range_size
                )
            )
        elif test_address >= range_end:
            self.fail(
                "address 0x{0:x} comes after range 0x{1:x} - 0x{2:x} (size 0x{3:x})".format(
                    test_address, range_start, range_end, range_size
                )
            )

    def add_threadinfo_collection_packets(self):
        self.test_sequence.add_log_lines(
            [
                {
                    "type": "multi_response",
                    "first_query": "qfThreadInfo",
                    "next_query": "qsThreadInfo",
                    "append_iteration_suffix": False,
                    "end_regex": re.compile(r"^\$(l)?#[0-9a-fA-F]{2}$"),
                    "save_key": "threadinfo_responses",
                }
            ],
            True,
        )

    def parse_threadinfo_packets(self, context):
        """Return an array of thread ids (decimal ints), one per thread."""
        threadinfo_responses = context.get("threadinfo_responses")
        self.assertIsNotNone(threadinfo_responses)

        thread_ids = []
        for threadinfo_response in threadinfo_responses:
            new_thread_infos = parse_threadinfo_response(threadinfo_response)
            thread_ids.extend(new_thread_infos)
        return thread_ids

    def launch_with_threads(self, thread_count):
        procs = self.prep_debug_monitor_and_inferior(
            inferior_args=["thread:new"] * (thread_count - 1) + ["trap"]
        )

        self.test_sequence.add_log_lines(
            [
                "read packet: $c#00",
                {
                    "direction": "send",
                    "regex": r"^\$T([0-9a-fA-F]{2})([^#]*)#..$",
                    "capture": {1: "stop_signo", 2: "stop_reply_kv"},
                },
            ],
            True,
        )
        self.add_threadinfo_collection_packets()
        context = self.expect_gdbremote_sequence()
        threads = self.parse_threadinfo_packets(context)
        self.assertGreaterEqual(len(threads), thread_count)
        return context, threads

    def add_set_breakpoint_packets(
        self, address, z_packet_type=0, do_continue=True, breakpoint_kind=1
    ):
        self.test_sequence.add_log_lines(
            [  # Set the breakpoint.
                "read packet: $Z{2},{0:x},{1}#00".format(
                    address, breakpoint_kind, z_packet_type
                ),
                # Verify the stub could set it.
                "send packet: $OK#00",
            ],
            True,
        )

        if do_continue:
            self.test_sequence.add_log_lines(
                [  # Continue the inferior.
                    "read packet: $c#63",
                    # Expect a breakpoint stop report.
                    {
                        "direction": "send",
                        "regex": r"^\$T([0-9a-fA-F]{2})thread:([0-9a-fA-F]+);",
                        "capture": {1: "stop_signo", 2: "stop_thread_id"},
                    },
                ],
                True,
            )

    def add_remove_breakpoint_packets(
        self, address, z_packet_type=0, breakpoint_kind=1
    ):
        self.test_sequence.add_log_lines(
            [  # Remove the breakpoint.
                "read packet: $z{2},{0:x},{1}#00".format(
                    address, breakpoint_kind, z_packet_type
                ),
                # Verify the stub could unset it.
                "send packet: $OK#00",
            ],
            True,
        )

    def add_qSupported_packets(self, client_features=[]):
        features = "".join(";" + x for x in client_features)
        self.test_sequence.add_log_lines(
            [
                "read packet: $qSupported{}#00".format(features),
                {
                    "direction": "send",
                    "regex": r"^\$(.*)#[0-9a-fA-F]{2}",
                    "capture": {1: "qSupported_response"},
                },
            ],
            True,
        )

    _KNOWN_QSUPPORTED_STUB_FEATURES = [
        "augmented-libraries-svr4-read",
        "PacketSize",
        "QStartNoAckMode",
        "QThreadSuffixSupported",
        "QListThreadsInStopReply",
        "qXfer:auxv:read",
        "qXfer:libraries:read",
        "qXfer:libraries-svr4:read",
        "qXfer:features:read",
        "qXfer:siginfo:read",
        "qEcho",
        "QPassSignals",
        "multiprocess",
        "fork-events",
        "vfork-events",
        "memory-tagging",
        "qSaveCore",
        "native-signals",
        "QNonStop",
        "SupportedWatchpointTypes",
        "SupportedCompressions",
        "MultiMemRead",
    ]

    def parse_qSupported_response(self, context):
        self.assertIsNotNone(context)

        raw_response = context.get("qSupported_response")
        self.assertIsNotNone(raw_response)

        # For values with key=val, the dict key and vals are set as expected.  For feature+, feature- and feature?, the
        # +,-,? is stripped from the key and set as the value.
        supported_dict = {}
        for match in re.finditer(r";?([^=;]+)(=([^;]+))?", raw_response):
            key = match.group(1)
            val = match.group(3)

            # key=val: store as is
            if val and len(val) > 0:
                supported_dict[key] = val
            else:
                if len(key) < 2:
                    raise Exception(
                        "singular stub feature is too short: must be stub_feature{+,-,?}"
                    )
                supported_type = key[-1]
                key = key[:-1]
                if not supported_type in ["+", "-", "?"]:
                    raise Exception(
                        "malformed stub feature: final character {} not in expected set (+,-,?)".format(
                            supported_type
                        )
                    )
                supported_dict[key] = supported_type
            # Ensure we know the supported element
            if key not in self._KNOWN_QSUPPORTED_STUB_FEATURES:
                raise Exception("unknown qSupported stub feature reported: %s" % key)

        return supported_dict

    def continue_process_and_wait_for_stop(self):
        self.test_sequence.add_log_lines(
            [
                "read packet: $vCont;c#a8",
                {
                    "direction": "send",
                    "regex": r"^\$T([0-9a-fA-F]{2})(.*)#[0-9a-fA-F]{2}$",
                    "capture": {1: "stop_signo", 2: "stop_key_val_text"},
                },
            ],
            True,
        )
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)
        return self.parse_interrupt_packets(context)

    def select_modifiable_register(self, reg_infos):
        """Find a register that can be read/written freely."""
        PREFERRED_REGISTER_NAMES = set(
            [
                "rax",
            ]
        )

        # First check for the first register from the preferred register name
        # set.
        alternative_register_index = None

        self.assertIsNotNone(reg_infos)
        for reg_info in reg_infos:
            if ("name" in reg_info) and (reg_info["name"] in PREFERRED_REGISTER_NAMES):
                # We found a preferred register.  Use it.
                return reg_info["lldb_register_index"]
            if ("generic" in reg_info) and (
                reg_info["generic"] == "fp" or reg_info["generic"] == "arg1"
            ):
                # A frame pointer or first arg register will do as a
                # register to modify temporarily.
                alternative_register_index = reg_info["lldb_register_index"]

        # We didn't find a preferred register.  Return whatever alternative register
        # we found, if any.
        return alternative_register_index

    def extract_registers_from_stop_notification(self, stop_key_vals_text):
        self.assertIsNotNone(stop_key_vals_text)
        kv_dict = self.parse_key_val_dict(stop_key_vals_text)

        registers = {}
        for key, val in list(kv_dict.items()):
            if re.match(r"^[0-9a-fA-F]+$", key):
                registers[int(key, 16)] = val
        return registers

    def gather_register_infos(self):
        self.reset_test_sequence()
        self.add_register_info_collection_packets()

        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        reg_infos = self.parse_register_info_packets(context)
        self.assertIsNotNone(reg_infos)
        self.add_lldb_register_index(reg_infos)

        return reg_infos

    def find_generic_register_with_name(self, reg_infos, generic_name):
        self.assertIsNotNone(reg_infos)
        for reg_info in reg_infos:
            if ("generic" in reg_info) and (reg_info["generic"] == generic_name):
                return reg_info
        return None

    def find_register_with_name_and_dwarf_regnum(self, reg_infos, name, dwarf_num):
        self.assertIsNotNone(reg_infos)
        for reg_info in reg_infos:
            if (reg_info["name"] == name) and (reg_info["dwarf"] == dwarf_num):
                return reg_info
        return None

    def decode_gdbremote_binary(self, encoded_bytes):
        decoded_bytes = ""
        i = 0
        while i < len(encoded_bytes):
            if encoded_bytes[i] == "}":
                # Handle escaped char.
                self.assertTrue(i + 1 < len(encoded_bytes))
                decoded_bytes += chr(ord(encoded_bytes[i + 1]) ^ 0x20)
                i += 2
            elif encoded_bytes[i] == "*":
                # Handle run length encoding.
                self.assertTrue(len(decoded_bytes) > 0)
                self.assertTrue(i + 1 < len(encoded_bytes))
                repeat_count = ord(encoded_bytes[i + 1]) - 29
                decoded_bytes += decoded_bytes[-1] * repeat_count
                i += 2
            else:
                decoded_bytes += encoded_bytes[i]
                i += 1
        return decoded_bytes

    def build_auxv_dict(self, endian, word_size, auxv_data):
        self.assertIsNotNone(endian)
        self.assertIsNotNone(word_size)
        self.assertIsNotNone(auxv_data)

        auxv_dict = {}

        # PowerPC64le's auxvec has a special key that must be ignored.
        # This special key may be used multiple times, resulting in
        # multiple key/value pairs with the same key, which would otherwise
        # break this test check for repeated keys.
        #
        # AT_IGNOREPPC = 22
        ignored_keys_for_arch = {"powerpc64le": [22]}
        arch = self.getArchitecture()
        ignore_keys = None
        if arch in ignored_keys_for_arch:
            ignore_keys = ignored_keys_for_arch[arch]

        while len(auxv_data) > 0:
            # Chop off key.
            raw_key = auxv_data[:word_size]
            auxv_data = auxv_data[word_size:]

            # Chop of value.
            raw_value = auxv_data[:word_size]
            auxv_data = auxv_data[word_size:]

            # Convert raw text from target endian.
            key = unpack_endian_binary_string(endian, raw_key)
            value = unpack_endian_binary_string(endian, raw_value)

            if ignore_keys and key in ignore_keys:
                continue

            # Handle ending entry.
            if key == 0:
                self.assertEqual(value, 0)
                return auxv_dict

            # The key should not already be present.
            self.assertFalse(key in auxv_dict)
            auxv_dict[key] = value

        self.fail(
            "should not reach here - implies required double zero entry not found"
        )
        return auxv_dict

    def read_binary_data_in_chunks(self, command_prefix, chunk_length):
        """Collect command_prefix{offset:x},{chunk_length:x} until a single 'l' or 'l' with data is returned."""
        offset = 0
        done = False
        decoded_data = ""

        while not done:
            # Grab the next iteration of data.
            self.reset_test_sequence()
            self.test_sequence.add_log_lines(
                [
                    "read packet: ${}{:x},{:x}:#00".format(
                        command_prefix, offset, chunk_length
                    ),
                    {
                        "direction": "send",
                        "regex": re.compile(
                            r"^\$([^E])(.*)#[0-9a-fA-F]{2}$", re.MULTILINE | re.DOTALL
                        ),
                        "capture": {1: "response_type", 2: "content_raw"},
                    },
                ],
                True,
            )

            context = self.expect_gdbremote_sequence()
            self.assertIsNotNone(context)

            response_type = context.get("response_type")
            self.assertIsNotNone(response_type)
            self.assertTrue(response_type in ["l", "m"])

            # Move offset along.
            offset += chunk_length

            # Figure out if we're done.  We're done if the response type is l.
            done = response_type == "l"

            # Decode binary data.
            content_raw = context.get("content_raw")
            if content_raw and len(content_raw) > 0:
                self.assertIsNotNone(content_raw)
                decoded_data += self.decode_gdbremote_binary(content_raw)
        return decoded_data

    def add_interrupt_packets(self):
        self.test_sequence.add_log_lines(
            [
                # Send the intterupt.
                "read packet: {}".format(chr(3)),
                # And wait for the stop notification.
                {
                    "direction": "send",
                    "regex": r"^\$T([0-9a-fA-F]{2})(.*)#[0-9a-fA-F]{2}$",
                    "capture": {1: "stop_signo", 2: "stop_key_val_text"},
                },
            ],
            True,
        )

    def parse_interrupt_packets(self, context):
        self.assertIsNotNone(context.get("stop_signo"))
        self.assertIsNotNone(context.get("stop_key_val_text"))
        return (
            int(context["stop_signo"], 16),
            self.parse_key_val_dict(context["stop_key_val_text"]),
        )

    def add_QSaveRegisterState_packets(self, thread_id):
        if thread_id:
            # Use the thread suffix form.
            request = "read packet: $QSaveRegisterState;thread:{:x}#00".format(
                thread_id
            )
        else:
            request = "read packet: $QSaveRegisterState#00"

        self.test_sequence.add_log_lines(
            [
                request,
                {
                    "direction": "send",
                    "regex": r"^\$(E?.*)#[0-9a-fA-F]{2}$",
                    "capture": {1: "save_response"},
                },
            ],
            True,
        )

    def parse_QSaveRegisterState_response(self, context):
        self.assertIsNotNone(context)

        save_response = context.get("save_response")
        self.assertIsNotNone(save_response)

        if len(save_response) < 1 or save_response[0] == "E":
            # error received
            return (False, None)
        else:
            return (True, int(save_response))

    def add_QRestoreRegisterState_packets(self, save_id, thread_id=None):
        if thread_id:
            # Use the thread suffix form.
            request = "read packet: $QRestoreRegisterState:{};thread:{:x}#00".format(
                save_id, thread_id
            )
        else:
            request = "read packet: $QRestoreRegisterState:{}#00".format(save_id)

        self.test_sequence.add_log_lines([request, "send packet: $OK#00"], True)

    def flip_all_bits_in_each_register_value(self, reg_infos, endian, thread_id=None):
        self.assertIsNotNone(reg_infos)

        successful_writes = 0
        failed_writes = 0

        for reg_info in reg_infos:
            # Use the lldb register index added to the reg info.  We're not necessarily
            # working off a full set of register infos, so an inferred register
            # index could be wrong.
            reg_index = reg_info["lldb_register_index"]
            self.assertIsNotNone(reg_index)

            reg_byte_size = int(reg_info["bitsize"]) // 8
            self.assertTrue(reg_byte_size > 0)

            # Handle thread suffix.
            if thread_id:
                p_request = "read packet: $p{:x};thread:{:x}#00".format(
                    reg_index, thread_id
                )
            else:
                p_request = "read packet: $p{:x}#00".format(reg_index)

            # Read the existing value.
            self.reset_test_sequence()
            self.test_sequence.add_log_lines(
                [
                    p_request,
                    {
                        "direction": "send",
                        "regex": r"^\$([0-9a-fA-F]+)#",
                        "capture": {1: "p_response"},
                    },
                ],
                True,
            )
            context = self.expect_gdbremote_sequence()
            self.assertIsNotNone(context)

            # Verify the response length.
            p_response = context.get("p_response")
            self.assertIsNotNone(p_response)
            initial_reg_value = unpack_register_hex_unsigned(endian, p_response)

            # Flip the value by xoring with all 1s
            all_one_bits_raw = "ff" * (int(reg_info["bitsize"]) // 8)
            flipped_bits_int = initial_reg_value ^ int(all_one_bits_raw, 16)
            # print("reg (index={}, name={}): val={}, flipped bits (int={}, hex={:x})".format(reg_index, reg_info["name"], initial_reg_value, flipped_bits_int, flipped_bits_int))

            # Handle thread suffix for P.
            if thread_id:
                P_request = "read packet: $P{:x}={};thread:{:x}#00".format(
                    reg_index,
                    pack_register_hex(
                        endian, flipped_bits_int, byte_size=reg_byte_size
                    ),
                    thread_id,
                )
            else:
                P_request = "read packet: $P{:x}={}#00".format(
                    reg_index,
                    pack_register_hex(
                        endian, flipped_bits_int, byte_size=reg_byte_size
                    ),
                )

            # Write the flipped value to the register.
            self.reset_test_sequence()
            self.test_sequence.add_log_lines(
                [
                    P_request,
                    {
                        "direction": "send",
                        "regex": r"^\$(OK|E[0-9a-fA-F]+)#[0-9a-fA-F]{2}",
                        "capture": {1: "P_response"},
                    },
                ],
                True,
            )
            context = self.expect_gdbremote_sequence()
            self.assertIsNotNone(context)

            # Determine if the write succeeded.  There are a handful of registers that can fail, or partially fail
            # (e.g. flags, segment selectors, etc.) due to register value restrictions.  Don't worry about them
            # all flipping perfectly.
            P_response = context.get("P_response")
            self.assertIsNotNone(P_response)
            if P_response == "OK":
                successful_writes += 1
            else:
                failed_writes += 1
                # print("reg (index={}, name={}) write FAILED (error: {})".format(reg_index, reg_info["name"], P_response))

            # Read back the register value, ensure it matches the flipped
            # value.
            if P_response == "OK":
                self.reset_test_sequence()
                self.test_sequence.add_log_lines(
                    [
                        p_request,
                        {
                            "direction": "send",
                            "regex": r"^\$([0-9a-fA-F]+)#",
                            "capture": {1: "p_response"},
                        },
                    ],
                    True,
                )
                context = self.expect_gdbremote_sequence()
                self.assertIsNotNone(context)

                verify_p_response_raw = context.get("p_response")
                self.assertIsNotNone(verify_p_response_raw)
                verify_bits = unpack_register_hex_unsigned(
                    endian, verify_p_response_raw
                )

                if verify_bits != flipped_bits_int:
                    # Some registers, like mxcsrmask and others, will permute what's written.  Adjust succeed/fail counts.
                    # print("reg (index={}, name={}): read verify FAILED: wrote {:x}, verify read back {:x}".format(reg_index, reg_info["name"], flipped_bits_int, verify_bits))
                    successful_writes -= 1
                    failed_writes += 1

        return (successful_writes, failed_writes)

    def is_bit_flippable_register(self, reg_info):
        if not reg_info:
            return False
        if not "set" in reg_info:
            return False
        if reg_info["set"] != "General Purpose Registers":
            return False
        if ("container-regs" in reg_info) and (len(reg_info["container-regs"]) > 0):
            # Don't try to bit flip registers contained in another register.
            return False
        if re.match("^.s$", reg_info["name"]):
            # This is a 2-letter register name that ends in "s", like a segment register.
            # Don't try to bit flip these.
            return False
        if re.match("^(c|)psr$", reg_info["name"]):
            # This is an ARM program status register; don't flip it.
            return False
        # Okay, this looks fine-enough.
        return True

    def read_register_values(self, reg_infos, endian, thread_id=None):
        self.assertIsNotNone(reg_infos)
        values = {}

        for reg_info in reg_infos:
            # We append a register index when load reg infos so we can work
            # with subsets.
            reg_index = reg_info.get("lldb_register_index")
            self.assertIsNotNone(reg_index)

            # Handle thread suffix.
            if thread_id:
                p_request = "read packet: $p{:x};thread:{:x}#00".format(
                    reg_index, thread_id
                )
            else:
                p_request = "read packet: $p{:x}#00".format(reg_index)

            # Read it with p.
            self.reset_test_sequence()
            self.test_sequence.add_log_lines(
                [
                    p_request,
                    {
                        "direction": "send",
                        "regex": r"^\$([0-9a-fA-F]+)#",
                        "capture": {1: "p_response"},
                    },
                ],
                True,
            )
            context = self.expect_gdbremote_sequence()
            self.assertIsNotNone(context)

            # Convert value from target endian to integral.
            p_response = context.get("p_response")
            self.assertIsNotNone(p_response)
            self.assertTrue(len(p_response) > 0)

            # on x86 Darwin, 4 GPR registers are often
            # unavailable, this is expected and correct.
            if (
                self.getArchitecture() == "x86_64"
                and self.platformIsDarwin()
                and p_response[0] == "E"
            ):
                values[reg_index] = 0
            else:
                self.assertFalse(p_response[0] == "E")

            values[reg_index] = unpack_register_hex_unsigned(endian, p_response)

        return values

    def add_vCont_query_packets(self):
        self.test_sequence.add_log_lines(
            [
                "read packet: $vCont?#49",
                {
                    "direction": "send",
                    "regex": r"^\$(vCont)?(.*)#[0-9a-fA-F]{2}$",
                    "capture": {2: "vCont_query_response"},
                },
            ],
            True,
        )

    def parse_vCont_query_response(self, context):
        self.assertIsNotNone(context)
        vCont_query_response = context.get("vCont_query_response")

        # Handle case of no vCont support at all - in which case the capture
        # group will be none or zero length.
        if not vCont_query_response or len(vCont_query_response) == 0:
            return {}

        return {
            key: 1 for key in vCont_query_response.split(";") if key and len(key) > 0
        }

    def count_single_steps_until_true(
        self,
        thread_id,
        predicate,
        args,
        max_step_count=100,
        use_Hc_packet=True,
        step_instruction="s",
    ):
        """Used by single step test that appears in a few different contexts."""
        single_step_count = 0

        while single_step_count < max_step_count:
            self.assertIsNotNone(thread_id)

            # Build the packet for the single step instruction.  We replace
            # {thread}, if present, with the thread_id.
            step_packet = "read packet: ${}#00".format(
                re.sub(r"{thread}", "{:x}".format(thread_id), step_instruction)
            )
            # print("\nstep_packet created: {}\n".format(step_packet))

            # Single step.
            self.reset_test_sequence()
            if use_Hc_packet:
                self.test_sequence.add_log_lines(
                    [  # Set the continue thread.
                        "read packet: $Hc{0:x}#00".format(thread_id),
                        "send packet: $OK#00",
                    ],
                    True,
                )
            self.test_sequence.add_log_lines(
                [
                    # Single step.
                    step_packet,
                    # "read packet: $vCont;s:{0:x}#00".format(thread_id),
                    # Expect a breakpoint stop report.
                    {
                        "direction": "send",
                        "regex": r"^\$T([0-9a-fA-F]{2})thread:([0-9a-fA-F]+);",
                        "capture": {1: "stop_signo", 2: "stop_thread_id"},
                    },
                ],
                True,
            )
            context = self.expect_gdbremote_sequence()
            self.assertIsNotNone(context)
            self.assertIsNotNone(context.get("stop_signo"))
            self.assertEqual(
                int(context.get("stop_signo"), 16),
                lldbutil.get_signal_number("SIGTRAP"),
            )

            single_step_count += 1

            # See if the predicate is true.  If so, we're done.
            if predicate(args):
                return (True, single_step_count)

        # The predicate didn't return true within the runaway step count.
        return (False, single_step_count)

    def g_c1_c2_contents_are(self, args):
        """Used by single step test that appears in a few different contexts."""
        g_c1_address = args["g_c1_address"]
        g_c2_address = args["g_c2_address"]
        expected_g_c1 = args["expected_g_c1"]
        expected_g_c2 = args["expected_g_c2"]

        # Read g_c1 and g_c2 contents.
        self.reset_test_sequence()
        self.test_sequence.add_log_lines(
            [
                "read packet: $m{0:x},{1:x}#00".format(g_c1_address, 1),
                {
                    "direction": "send",
                    "regex": r"^\$(.+)#[0-9a-fA-F]{2}$",
                    "capture": {1: "g_c1_contents"},
                },
                "read packet: $m{0:x},{1:x}#00".format(g_c2_address, 1),
                {
                    "direction": "send",
                    "regex": r"^\$(.+)#[0-9a-fA-F]{2}$",
                    "capture": {1: "g_c2_contents"},
                },
            ],
            True,
        )

        # Run the packet stream.
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Check if what we read from inferior memory is what we are expecting.
        self.assertIsNotNone(context.get("g_c1_contents"))
        self.assertIsNotNone(context.get("g_c2_contents"))

        return (seven.unhexlify(context.get("g_c1_contents")) == expected_g_c1) and (
            seven.unhexlify(context.get("g_c2_contents")) == expected_g_c2
        )

    def single_step_only_steps_one_instruction(
        self, use_Hc_packet=True, step_instruction="s"
    ):
        """Used by single step test that appears in a few different contexts."""
        # Start up the inferior.
        procs = self.prep_debug_monitor_and_inferior(
            inferior_args=[
                "get-code-address-hex:swap_chars",
                "get-data-address-hex:g_c1",
                "get-data-address-hex:g_c2",
                "sleep:1",
                "call-function:swap_chars",
                "sleep:5",
            ]
        )

        # Run the process
        self.test_sequence.add_log_lines(
            [  # Start running after initial stop.
                "read packet: $c#63",
                # Match output line that prints the memory address of the function call entry point.
                # Note we require launch-only testing so we can get inferior otuput.
                {
                    "type": "output_match",
                    "regex": r"^code address: 0x([0-9a-fA-F]+)\r\ndata address: 0x([0-9a-fA-F]+)\r\ndata address: 0x([0-9a-fA-F]+)\r\n$",
                    "capture": {
                        1: "function_address",
                        2: "g_c1_address",
                        3: "g_c2_address",
                    },
                },
                # Now stop the inferior.
                "read packet: {}".format(chr(3)),
                # And wait for the stop notification.
                {
                    "direction": "send",
                    "regex": r"^\$T([0-9a-fA-F]{2})thread:([0-9a-fA-F]+);",
                    "capture": {1: "stop_signo", 2: "stop_thread_id"},
                },
            ],
            True,
        )

        # Run the packet stream.
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Grab the main thread id.
        self.assertIsNotNone(context.get("stop_thread_id"))
        main_thread_id = int(context.get("stop_thread_id"), 16)

        # Grab the function address.
        self.assertIsNotNone(context.get("function_address"))
        function_address = int(context.get("function_address"), 16)

        # Grab the data addresses.
        self.assertIsNotNone(context.get("g_c1_address"))
        g_c1_address = int(context.get("g_c1_address"), 16)

        self.assertIsNotNone(context.get("g_c2_address"))
        g_c2_address = int(context.get("g_c2_address"), 16)

        # Set a breakpoint at the given address.
        if self.getArchitecture().startswith("arm"):
            # TODO: Handle case when setting breakpoint in thumb code
            BREAKPOINT_KIND = 4
        else:
            BREAKPOINT_KIND = 1
        self.reset_test_sequence()
        self.add_set_breakpoint_packets(
            function_address, do_continue=True, breakpoint_kind=BREAKPOINT_KIND
        )
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Remove the breakpoint.
        self.reset_test_sequence()
        self.add_remove_breakpoint_packets(
            function_address, breakpoint_kind=BREAKPOINT_KIND
        )
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Verify g_c1 and g_c2 match expected initial state.
        args = {}
        args["g_c1_address"] = g_c1_address
        args["g_c2_address"] = g_c2_address
        args["expected_g_c1"] = "0"
        args["expected_g_c2"] = "1"

        self.assertTrue(self.g_c1_c2_contents_are(args))

        # Verify we take only a small number of steps to hit the first state.
        # Might need to work through function entry prologue code.
        args["expected_g_c1"] = "1"
        args["expected_g_c2"] = "1"
        (state_reached, step_count) = self.count_single_steps_until_true(
            main_thread_id,
            self.g_c1_c2_contents_are,
            args,
            max_step_count=25,
            use_Hc_packet=use_Hc_packet,
            step_instruction=step_instruction,
        )
        self.assertTrue(state_reached)

        # Verify we hit the next state.
        args["expected_g_c1"] = "1"
        args["expected_g_c2"] = "0"
        (state_reached, step_count) = self.count_single_steps_until_true(
            main_thread_id,
            self.g_c1_c2_contents_are,
            args,
            max_step_count=5,
            use_Hc_packet=use_Hc_packet,
            step_instruction=step_instruction,
        )
        self.assertTrue(state_reached)
        expected_step_count = 1
        arch = self.getArchitecture()

        # MIPS required "3" (ADDIU, SB, LD) machine instructions for updation
        # of variable value
        if re.match("mips", arch):
            expected_step_count = 3
        # S390X requires "2" (LARL, MVI) machine instructions for updation of
        # variable value
        if re.match("s390x", arch):
            expected_step_count = 2
        # ARM64 requires "4" instructions: 2 to compute the address (adrp,
        # add), one to materialize the constant (mov) and the store. Once
        # addresses and constants are materialized, only one instruction is
        # needed.
        if re.match("arm64", arch):
            before_materialization_step_count = 4
            after_matrialization_step_count = 1
            self.assertIn(
                step_count,
                [before_materialization_step_count, after_matrialization_step_count],
            )
            expected_step_count = after_matrialization_step_count
        else:
            self.assertEqual(step_count, expected_step_count)

        # Verify we hit the next state.
        args["expected_g_c1"] = "0"
        args["expected_g_c2"] = "0"
        (state_reached, step_count) = self.count_single_steps_until_true(
            main_thread_id,
            self.g_c1_c2_contents_are,
            args,
            max_step_count=5,
            use_Hc_packet=use_Hc_packet,
            step_instruction=step_instruction,
        )
        self.assertTrue(state_reached)
        self.assertEqual(step_count, expected_step_count)

        # Verify we hit the next state.
        args["expected_g_c1"] = "0"
        args["expected_g_c2"] = "1"
        (state_reached, step_count) = self.count_single_steps_until_true(
            main_thread_id,
            self.g_c1_c2_contents_are,
            args,
            max_step_count=5,
            use_Hc_packet=use_Hc_packet,
            step_instruction=step_instruction,
        )
        self.assertTrue(state_reached)
        self.assertEqual(step_count, expected_step_count)

    def maybe_strict_output_regex(self, regex):
        return (
            ".*" + regex + ".*"
            if lldbplatformutil.hasChattyStderr(self)
            else "^" + regex + "$"
        )

    def install_and_create_launch_args(self):
        exe_path = self.getBuildArtifact("a.out")
        if not lldb.remote_platform:
            return [exe_path]
        remote_path = lldbutil.append_to_process_working_directory(
            self, os.path.basename(exe_path)
        )
        remote_file_spec = lldb.SBFileSpec(remote_path, False)
        err = lldb.remote_platform.Install(
            lldb.SBFileSpec(exe_path, True), remote_file_spec
        )
        if err.Fail():
            raise Exception(
                "remote_platform.Install('%s', '%s') failed: %s"
                % (exe_path, remote_path, err)
            )
        return [remote_path]



#!/usr/bin/env python3
import argparse
import os
import re
import shlex
import subprocess
import sys
import textwrap

msg_prefix = "\n> NFC-Mode:"

def get_relevant_bolt_changes(dir: str) -> str:
    # Return a list of bolt source changes that are relevant to testing.
    all_changes = subprocess.run(
        shlex.split("git show HEAD --name-only --pretty=''"),
        cwd=dir,
        text=True,
        stdout=subprocess.PIPE,
    )
    keep_bolt = subprocess.run(
        shlex.split("grep '^bolt'"),
        input=all_changes.stdout,
        text=True,
        stdout=subprocess.PIPE,
    )
    keep_relevant = subprocess.run(
        shlex.split(
            "grep -v -e '^bolt/docs' -e '^bolt/utils/docker' -e '^bolt/utils/dot2html'"
        ),
        input=keep_bolt.stdout,
        text=True,
        stdout=subprocess.PIPE,
    )
    return keep_relevant.stdout

def get_git_ref_or_rev(dir: str) -> str:
    # Run 'git symbolic-ref -q --short HEAD || git rev-parse --short HEAD'
    cmd_ref = "git symbolic-ref -q --short HEAD"
    ref = subprocess.run(
        shlex.split(cmd_ref), cwd=dir, text=True, stdout=subprocess.PIPE
    )
    if not ref.returncode:
        return ref.stdout.strip()
    cmd_rev = "git rev-parse --short HEAD"
    return subprocess.check_output(shlex.split(cmd_rev), cwd=dir, text=True).strip()

def switch_back(
    switch_back: bool, stash: bool, source_dir: str, old_ref: str, new_ref: str
):
    # Switch back to the current revision if needed and inform the user of where
    # the HEAD is. Must be called after checking out the previous commit on all
    # exit paths.
    if switch_back:
        print(f"{msg_prefix} Switching back to current revision..")
        if stash:
            subprocess.run(shlex.split("git stash pop"), cwd=source_dir)
        subprocess.run(shlex.split(f"git checkout {old_ref}"), cwd=source_dir)
    else:
        print(
            f"The repository {source_dir} has been switched from {old_ref} "
            f"to {new_ref}. Local changes were stashed. Switch back using\n\t"
            f"git checkout {old_ref}\n"
        )

def main():
    parser = argparse.ArgumentParser(
        description=textwrap.dedent(
            """
            This script builds two versions of BOLT:
            llvm-bolt.new, using the current revision, and llvm-bolt.old using
            the previous revision. These can be used to check whether the
            current revision changes BOLT's functional behavior.
            """
        )
    )
    parser.add_argument(
        "build_dir",
        nargs="?",
        default=os.getcwd(),
        help="Path to BOLT build directory, default is current " "directory",
    )
    parser.add_argument(
        "--create-wrapper",
        default=False,
        action="store_true",
        help="Sets up llvm-bolt as a symlink to llvm-bolt-wrapper. Passes the options through to llvm-bolt-wrapper.",
    )
    parser.add_argument(
        "--check-bolt-sources",
        default=False,
        action="store_true",
        help="Create a marker file (.llvm-bolt.changes) if any relevant BOLT sources are modified",
    )
    parser.add_argument(
        "--switch-back",
        default=False,
        action="store_true",
        help="Checkout back to the starting revision",
    )
    parser.add_argument(
        "--cmp-rev",
        default="HEAD^",
        help="Revision to checkout to compare vs HEAD",
    )

    # When creating a wrapper, pass any unknown arguments to it. Otherwise, die.
    args, wrapper_args = parser.parse_known_args()
    if not args.create_wrapper and len(wrapper_args) > 0:
        parser.parse_args()

    # Find the repo directory.
    source_dir = None
    try:
        CMCacheFilename = f"{args.build_dir}/CMakeCache.txt"
        with open(CMCacheFilename) as f:
            for line in f:
                m = re.match(r"LLVM_SOURCE_DIR:STATIC=(.*)", line)
                if m:
                    source_dir = m.groups()[0]
        if not source_dir:
            raise Exception(f"Source directory not found: '{CMCacheFilename}'")
    except Exception as e:
        sys.exit(e)

    # Clean the previous llvm-bolt if it exists.
    bolt_path = f"{args.build_dir}/bin/llvm-bolt"
    if os.path.exists(bolt_path):
        os.remove(bolt_path)

    # Build the current commit.
    print(f"{msg_prefix} Building current revision..")
    subprocess.run(
        shlex.split("cmake --build . --target llvm-bolt"), cwd=args.build_dir
    )

    if not os.path.exists(bolt_path):
        sys.exit(f"Failed to build the current revision: '{bolt_path}'")

    # Rename llvm-bolt and memorize the old hash for logging.
    os.replace(bolt_path, f"{bolt_path}.new")
    old_ref = get_git_ref_or_rev(source_dir)

    if args.check_bolt_sources:
        marker = f"{args.build_dir}/.llvm-bolt.changes"
        if os.path.exists(marker):
            os.remove(marker)
        file_changes = get_relevant_bolt_changes(source_dir)
        # Create a marker file if any relevant BOLT source files changed.
        if len(file_changes) > 0:
            print(f"BOLT source changes were found:\n{file_changes}")
            open(marker, "a").close()

    # Determine whether a stash is needed.
    stash = subprocess.run(
        shlex.split("git status --porcelain"),
        cwd=source_dir,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=True,
    ).stdout
    if stash:
        # Save local changes before checkout.
        subprocess.run(shlex.split("git stash push -u"), cwd=source_dir)

    # Check out the previous/cmp commit and get its commit hash for logging.
    subprocess.run(shlex.split(f"git checkout -f {args.cmp_rev}"), cwd=source_dir)
    new_ref = get_git_ref_or_rev(source_dir)

    # Build the previous commit.
    print(f"{msg_prefix} Building previous revision..")
    subprocess.run(
        shlex.split("cmake --build . --target llvm-bolt"), cwd=args.build_dir
    )

    # Rename llvm-bolt.
    if not os.path.exists(bolt_path):
        print(f"Failed to build the previous revision: '{bolt_path}'")
        switch_back(args.switch_back, stash, source_dir, old_ref, new_ref)
        sys.exit(1)
    os.replace(bolt_path, f"{bolt_path}.old")

    # Symlink llvm-bolt-wrapper
    if args.create_wrapper:
        print(f"{msg_prefix} Creating llvm-bolt wrapper..")
        script_dir = os.path.dirname(os.path.abspath(__file__))
        wrapper_path = f"{script_dir}/llvm-bolt-wrapper.py"
        try:
            # Set up llvm-bolt-wrapper.ini
            ini = subprocess.check_output(
                shlex.split(f"{wrapper_path} {bolt_path}.old {bolt_path}.new")
                + wrapper_args,
                text=True,
            )
            with open(f"{args.build_dir}/bin/llvm-bolt-wrapper.ini", "w") as f:
                f.write(ini)
            os.symlink(wrapper_path, bolt_path)
        except Exception as e:
            print("Failed to create a wrapper:\n" + str(e))
            switch_back(args.switch_back, stash, source_dir, old_ref, new_ref)
            sys.exit(1)

    switch_back(args.switch_back, stash, source_dir, old_ref, new_ref)

    print(
        f"{msg_prefix} Completed!\nBuild directory {args.build_dir} is ready for"
        " NFC-Mode comparison between the two revisions."
    )

    if args.create_wrapper:
        print(
            "Can run BOLT tests using:\n"
            "\tbin/llvm-lit -sv tools/bolt/test\nor\n"
            "\tbin/llvm-lit -sv tools/bolttests"
        )


if __name__ == "__main__":
    main()



"""
Test lldb log handlers.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LogHandlerTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.log_file = self.getBuildArtifact("log-file.txt")
        if os.path.exists(self.log_file):
            os.remove(self.log_file)

    def test_circular(self):
        self.runCmd("log enable -b 5 -h circular lldb commands")
        self.runCmd("bogus", check=False)
        self.runCmd("log dump lldb -f {}".format(self.log_file))

        with open(self.log_file, "r") as f:
            log_lines = f.readlines()

        self.assertEqual(len(log_lines), 5)

        found_command_log_dump = False
        found_command_bogus = False

        for line in log_lines:
            if "Processing command: log dump" in line:
                found_command_log_dump = True
            if "Processing command: bogus" in line:
                found_command_bogus = True

        self.assertTrue(found_command_log_dump)
        self.assertFalse(found_command_bogus)

    def test_circular_no_buffer_size(self):
        self.expect(
            "log enable -h circular lldb commands",
            error=True,
            substrs=["the circular buffer handler requires a non-zero buffer size"],
        )

    def test_dump_unsupported(self):
        self.runCmd("log enable lldb commands -f {}".format(self.log_file))
        self.expect(
            "log dump lldb",
            error=True,
            substrs=["log channel 'lldb' does not support dumping"],
        )



"""
Test the SB API SBFrame::GuessLanguage.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestFrameGuessLanguage(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    @skipIf(compiler="clang", compiler_version=["<", "10.0"])
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr37658")
    def test_guess_language(self):
        """Test GuessLanguage for C and C++."""
        self.build()
        self.do_test()

    def check_language(self, thread, frame_no, test_lang):
        frame = thread.frames[frame_no]
        self.assertTrue(frame.IsValid(), "Frame %d was not valid." % (frame_no))
        lang = frame.GuessLanguage()
        self.assertEqual(lang, test_lang)

    def do_test(self):
        """Test GuessLanguage for C & C++."""
        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "Set breakpoint here", lldb.SBFileSpec("somefunc.c")
        )

        c_frame_language = lldb.eLanguageTypeC99
        cxx_frame_language = lldb.eLanguageTypeC_plus_plus_11
        # gcc emits DW_LANG_C89 even if -std=c99 was specified
        if "gcc" in self.getCompiler():
            c_frame_language = lldb.eLanguageTypeC89
            cxx_frame_language = lldb.eLanguageTypeC_plus_plus

        self.check_language(thread, 0, c_frame_language)
        self.check_language(thread, 1, cxx_frame_language)
        self.check_language(thread, 2, lldb.eLanguageTypeC_plus_plus)



import collections
import os
import re
import operator

import lit.Test
import lit.TestRunner
import lit.util
from lit.formats.base import TestFormat


class LLDBTest(TestFormat):
    def __init__(self, dotest_cmd):
        self.dotest_cmd = dotest_cmd

    def getTestsInDirectory(self, testSuite, path_in_suite, litConfig, localConfig):
        source_path = testSuite.getSourcePath(path_in_suite)
        for filename in os.listdir(source_path):
            # Ignore dot files and excluded tests.
            if filename.startswith(".") or filename in localConfig.excludes:
                continue

            # Ignore files that don't start with 'Test'.
            if not filename.startswith("Test"):
                continue

            filepath = os.path.join(source_path, filename)
            if not os.path.isdir(filepath):
                base, ext = os.path.splitext(filename)
                if ext in localConfig.suffixes:
                    yield lit.Test.Test(
                        testSuite, path_in_suite + (filename,), localConfig
                    )

    def execute(self, test, litConfig):
        if litConfig.noExecute:
            return lit.Test.PASS, ""

        if not getattr(test.config, "lldb_enable_python", False):
            return (lit.Test.UNSUPPORTED, "Python module disabled")

        if test.config.unsupported:
            return (lit.Test.UNSUPPORTED, "Test is unsupported")

        testPath, testFile = os.path.split(test.getSourcePath())

        # The Python used to run lit can be different from the Python LLDB was
        # build with.
        executable = test.config.python_executable

        isLuaTest = testFile == test.config.lua_test_entry

        # On Windows, the system does not always correctly interpret
        # shebang lines.  To make sure we can execute the tests, add
        # python exe as the first parameter of the command.
        cmd = [executable] + self.dotest_cmd + [testPath, "-p", testFile]

        launcher = getattr(test.config, "lldb_launcher", None)
        if launcher:
            cmd = [launcher] + cmd

        if isLuaTest:
            cmd.extend(["--env", "LUA_EXECUTABLE=%s" % test.config.lua_executable])
            cmd.extend(["--env", "LLDB_LUA_CPATH=%s" % test.config.lldb_lua_cpath])

        timeoutInfo = None
        try:
            out, err, exitCode = lit.util.executeCommand(
                cmd,
                env=test.config.environment,
                timeout=litConfig.maxIndividualTestTime,
            )
        except lit.util.ExecuteCommandTimeoutException as e:
            out = e.out
            err = e.err
            exitCode = e.exitCode
            timeoutInfo = "Reached timeout of {} seconds".format(
                litConfig.maxIndividualTestTime
            )

        output = """Script:\n--\n%s\n--\nExit Code: %d\n""" % (" ".join(cmd), exitCode)
        if timeoutInfo is not None:
            output += """Timeout: %s\n""" % (timeoutInfo,)
        output += "\n"

        if out:
            output += """Command Output (stdout):\n--\n%s\n--\n""" % (out,)
        if err:
            output += """Command Output (stderr):\n--\n%s\n--\n""" % (err,)

        if timeoutInfo:
            return lit.Test.TIMEOUT, output

        # Parse the dotest output from stderr. First get the # of total tests, in order to infer the # of passes.
        # Example: "Ran 5 tests in 0.042s"
        num_ran_regex = r"^Ran (\d+) tests? in "
        num_ran_results = re.search(num_ran_regex, err, re.MULTILINE)

        # If parsing fails mark this test as unresolved.
        if not num_ran_results:
            return lit.Test.UNRESOLVED, output
        num_ran = int(num_ran_results.group(1))

        # Then look for a detailed summary, which is OK or FAILED followed by optional details.
        # Example: "OK (skipped=1, expected failures=1)"
        # Example: "FAILED (failures=3)"
        # Example: "OK"
        result_regex = r"^(?:OK|FAILED)(?: \((.*)\))?\r?$"
        results = re.search(result_regex, err, re.MULTILINE)

        # If parsing fails mark this test as unresolved.
        if not results:
            return lit.Test.UNRESOLVED, output

        details = results.group(1)
        parsed_details = collections.defaultdict(int)
        if details:
            for detail in details.split(", "):
                detail_parts = detail.split("=")
                if len(detail_parts) != 2:
                    return lit.Test.UNRESOLVED, output
                parsed_details[detail_parts[0]] = int(detail_parts[1])

        failures = parsed_details["failures"]
        errors = parsed_details["errors"]
        skipped = parsed_details["skipped"]
        expected_failures = parsed_details["expected failures"]
        unexpected_successes = parsed_details["unexpected successes"]

        non_pass = (
            failures + errors + skipped + expected_failures + unexpected_successes
        )
        passes = num_ran - non_pass

        if exitCode:
            # Aggregate the tests results with the following precedence:
            # UNRESOLVED > FAIL > XPASS
            if errors > 0:
                return lit.Test.UNRESOLVED, output
            if failures > 0:
                return lit.Test.FAIL, output
            return lit.Test.XPASS, output
        else:
            # Aggregate the tests results with the following precedence:
            # PASS > XFAIL > UNSUPPORTED
            if passes > 0:
                return lit.Test.PASS, output
            if expected_failures > 0:
                return lit.Test.XFAIL, output
            return lit.Test.UNSUPPORTED, output



"""
Make sure that the concurrent vfork() from multiple threads works correctly.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestConcurrentVFork(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def build_run_to_breakpoint(self, use_fork, call_exec):
        self.build()

        args = []
        if use_fork:
            args.append("--fork")
        if call_exec:
            args.append("--exec")
        launch_info = lldb.SBLaunchInfo(args)
        launch_info.SetWorkingDirectory(self.getBuildDir())

        return lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

    def follow_parent_helper(self, use_fork, call_exec):
        (target, process, thread, bkpt) = self.build_run_to_breakpoint(
            use_fork, call_exec
        )

        parent_pid = target.FindFirstGlobalVariable("g_pid").GetValueAsUnsigned()
        self.runCmd("settings set target.process.follow-fork-mode parent")
        self.runCmd("settings set target.process.stop-on-exec False", check=False)
        self.expect(
            "continue", substrs=[f"Process {parent_pid} exited with status = 0"]
        )

    def follow_child_helper(self, use_fork, call_exec):
        self.build_run_to_breakpoint(use_fork, call_exec)

        self.runCmd("settings set target.process.follow-fork-mode child")
        self.runCmd("settings set target.process.stop-on-exec False", check=False)
        # Child process exits with code "index + 10" since index is [0-4]
        # so the exit code should be 1[0-4]
        self.expect("continue", patterns=[r"exited with status = 1[0-4]"])

    @skipUnlessPlatform(["linux"])
    # https://github.com/llvm/llvm-project/issues/85084.
    @skipIf(oslist=["linux"])
    def test_follow_parent_vfork_no_exec(self):
        """
        Make sure that debugging concurrent vfork() from multiple threads won't crash lldb during follow-parent.
        And follow-parent successfully detach all child processes and exit debugger without calling exec.
        """
        self.follow_parent_helper(use_fork=False, call_exec=False)

    @skipUnlessPlatform(["linux"])
    # https://github.com/llvm/llvm-project/issues/85084.
    @skipIf(oslist=["linux"])
    def test_follow_parent_fork_no_exec(self):
        """
        Make sure that debugging concurrent fork() from multiple threads won't crash lldb during follow-parent.
        And follow-parent successfully detach all child processes and exit debugger without calling exec
        """
        self.follow_parent_helper(use_fork=True, call_exec=False)

    @skipUnlessPlatform(["linux"])
    # https://github.com/llvm/llvm-project/issues/85084.
    @skipIf(oslist=["linux"])
    def test_follow_parent_vfork_call_exec(self):
        """
        Make sure that debugging concurrent vfork() from multiple threads won't crash lldb during follow-parent.
        And follow-parent successfully detach all child processes and exit debugger after calling exec.
        """
        self.follow_parent_helper(use_fork=False, call_exec=True)

    @skipUnlessPlatform(["linux"])
    # https://github.com/llvm/llvm-project/issues/85084.
    @skipIf(oslist=["linux"])
    def test_follow_parent_fork_call_exec(self):
        """
        Make sure that debugging concurrent vfork() from multiple threads won't crash lldb during follow-parent.
        And follow-parent successfully detach all child processes and exit debugger after calling exec.
        """
        self.follow_parent_helper(use_fork=True, call_exec=True)

    @skipUnlessPlatform(["linux"])
    # https://github.com/llvm/llvm-project/issues/85084.
    @skipIf(oslist=["linux"])
    def test_follow_child_vfork_no_exec(self):
        """
        Make sure that debugging concurrent vfork() from multiple threads won't crash lldb during follow-child.
        And follow-child successfully detach parent process and exit child process with correct exit code without calling exec.
        """
        self.follow_child_helper(use_fork=False, call_exec=False)

    @skipUnlessPlatform(["linux"])
    # https://github.com/llvm/llvm-project/issues/85084.
    @skipIf(oslist=["linux"])
    def test_follow_child_fork_no_exec(self):
        """
        Make sure that debugging concurrent fork() from multiple threads won't crash lldb during follow-child.
        And follow-child successfully detach parent process and exit child process with correct exit code without calling exec.
        """
        self.follow_child_helper(use_fork=True, call_exec=False)

    @skipUnlessPlatform(["linux"])
    # https://github.com/llvm/llvm-project/issues/85084.
    @skipIf(oslist=["linux"])
    def test_follow_child_vfork_call_exec(self):
        """
        Make sure that debugging concurrent vfork() from multiple threads won't crash lldb during follow-child.
        And follow-child successfully detach parent process and exit child process with correct exit code after calling exec.
        """
        self.follow_child_helper(use_fork=False, call_exec=True)

    @skipUnlessPlatform(["linux"])
    # https://github.com/llvm/llvm-project/issues/85084.
    @skipIf(oslist=["linux"])
    def test_follow_child_fork_call_exec(self):
        """
        Make sure that debugging concurrent fork() from multiple threads won't crash lldb during follow-child.
        And follow-child successfully detach parent process and exit child process with correct exit code after calling exec.
        """
        self.follow_child_helper(use_fork=True, call_exec=True)



"""Test that we handle inferiors which change their process group"""


import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ChangeProcessGroupTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.c.
        self.line = line_number("main.c", "// Set breakpoint here")

    @skipIfFreeBSD  # Times out on FreeBSD llvm.org/pr23731
    @skipIfWindows  # setpgid call does not exist on Windows
    @expectedFailureAndroid("http://llvm.org/pr23762", api_levels=[16])
    @expectedFailureNetBSD
    @skipIftvOS  # fork not available on tvOS.
    @skipIfwatchOS  # fork not available on watchOS.
    def test_setpgid(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Use a file as a synchronization point between test and inferior.
        pid_file_path = lldbutil.append_to_process_working_directory(
            self, "pid_file_%d" % (int(time.time()))
        )
        self.addTearDownHook(
            lambda: self.run_platform_command("rm %s" % (pid_file_path))
        )

        popen = self.spawnSubprocess(exe, [pid_file_path])

        pid = lldbutil.wait_for_file_on_target(self, pid_file_path)

        # make sure we cleanup the forked child also
        def cleanupChild():
            if lldb.remote_platform:
                lldb.remote_platform.Kill(int(pid))
            else:
                if os.path.exists("/proc/" + pid):
                    os.kill(int(pid), signal.SIGKILL)

        self.addTearDownHook(cleanupChild)

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        listener = lldb.SBListener("my.attach.listener")
        error = lldb.SBError()
        process = target.AttachToProcessWithID(listener, int(pid), error)
        self.assertTrue(error.Success() and process, PROCESS_IS_VALID)

        # set a breakpoint just before the setpgid() call
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=-1
        )

        thread = process.GetSelectedThread()

        # release the child from its loop
        value = thread.GetSelectedFrame().EvaluateExpression("release_child_flag = 1")
        self.assertTrue(value.IsValid())
        self.assertEqual(value.GetValueAsUnsigned(0), 1)
        process.Continue()

        # make sure the child's process group id is different from its pid
        value = thread.GetSelectedFrame().EvaluateExpression("(int)getpgid(0)")
        self.assertTrue(value.IsValid())
        self.assertNotEqual(value.GetValueAsUnsigned(0), int(pid))

        # step over the setpgid() call
        thread.StepOver()
        self.assertStopReason(thread.GetStopReason(), lldb.eStopReasonPlanComplete)

        # verify that the process group has been set correctly
        # this also checks that we are still in full control of the child
        value = thread.GetSelectedFrame().EvaluateExpression("(int)getpgid(0)")
        self.assertTrue(value.IsValid())
        self.assertEqual(value.GetValueAsUnsigned(0), int(pid))

        # run to completion
        process.Continue()
        self.assertState(process.GetState(), lldb.eStateExited)



"""
Test 'breakpoint command list'.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test_list_commands(self):
        src_dir = self.getSourceDir()
        yaml_path = os.path.join(src_dir, "a.yaml")
        yaml_base, ext = os.path.splitext(yaml_path)
        obj_path = self.getBuildArtifact("main.o")
        self.yaml2obj(yaml_path, obj_path)

        # Create a target with the object file we just created from YAML
        target = self.dbg.CreateTarget(obj_path)
        self.assertTrue(target, VALID_TARGET)

        # Test without any breakpoints.
        self.expect(
            "breakpoint command list 1",
            error=True,
            substrs=["error: No breakpoints exist for which to list commands"],
        )

        # Set a breakpoint
        self.runCmd("b foo")

        # Check list breakpoint commands for breakpoints that have no commands.
        self.expect(
            "breakpoint command list 1",
            startstr="Breakpoint 1 does not have an associated command.",
        )

        # Add a breakpoint command.
        self.runCmd("breakpoint command add -o 'source list' 1")

        # List breakpoint command that we just created.
        self.expect(
            "breakpoint command list 1",
            startstr="""Breakpoint 1:
    Breakpoint commands:
      source list
""",
        )

        # List breakpoint command with invalid breakpoint ID.
        self.expect(
            "breakpoint command list 2",
            error=True,
            startstr="error: '2' is not a currently valid breakpoint ID.",
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects.openmp import *


def constructAndPrintInModule(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f()
        print(module)
    return f


# CHECK-LABEL: test_barrier
# CHECK: module {
# CHECK:   omp.barrier
# CHECK: }
@constructAndPrintInModule
def test_barrier():
    barrier = BarrierOp()



"""
Watch one byte in the middle of a doubleword, mutate the
entire doubleword including the watched byte.  On AArch64
the trap address is probably the start of the doubleword,
instead of the address of our watched byte.  Test that lldb
correctly associates this watchpoint trap with our watchpoint.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class UnalignedWatchpointTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_unaligned_watchpoint(self):
        """Test an unaligned watchpoint triggered by a larger aligned write."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", self.main_source_file
        )

        frame = thread.GetFrameAtIndex(0)

        self.expect("watchpoint set variable a.buf[2]")

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (write type), but
        # only once.  The stop reason of the thread should be watchpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=["stopped", "stop reason = watchpoint"],
        )

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should now be 1.
        self.expect("watchpoint list -v", substrs=["hit_count = 1"])

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (write type), but
        # only once.  The stop reason of the thread should be watchpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=["stopped", "stop reason = watchpoint"],
        )

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should now be 1.
        self.expect("watchpoint list -v", substrs=["hit_count = 2"])



"""
Use lldb Python SBTarget API to set read watchpoints
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SetReadOnlyWatchpointTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Our simple source filename.
        self.source = "main.c"
        # Find the line number to break inside main().
        self.line = line_number(self.source, "// Set break point at this line.")
        self.build()

    # Intel hardware does not support read-only watchpoints
    @expectedFailureAll(archs=["i386", "x86_64"])
    def test_read_watchpoint_watch_address(self):
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c.
        breakpoint = target.BreakpointCreateByLocation(self.source, self.line)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        # We should be stopped due to the breakpoint.  Get frame #0.
        process = target.GetProcess()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        frame0 = thread.GetFrameAtIndex(0)

        value = frame0.FindValue("global", lldb.eValueTypeVariableGlobal)
        local = frame0.FindValue("local", lldb.eValueTypeVariableLocal)
        error = lldb.SBError()

        watchpoint = target.WatchAddress(value.GetLoadAddress(), 1, True, False, error)
        self.assertTrue(
            value and local and watchpoint,
            "Successfully found the values and set a watchpoint",
        )
        self.DebugSBValue(value)
        self.DebugSBValue(local)

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        print(watchpoint)

        # Continue.  Expect the program to stop due to the variable being
        # read, but *not* written to.
        process.Continue()

        if self.TraceOn():
            lldbutil.print_stacktraces(process)

        self.assertTrue(
            local.GetValueAsSigned() > 0, "The local variable has been incremented"
        )

    # Intel hardware does not support read-only watchpoints
    @expectedFailureAll(archs=["i386", "x86_64"])
    def test_read_watchpoint_watch_create_by_address(self):
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c.
        breakpoint = target.BreakpointCreateByLocation(self.source, self.line)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        # We should be stopped due to the breakpoint.  Get frame #0.
        process = target.GetProcess()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        frame0 = thread.GetFrameAtIndex(0)

        value = frame0.FindValue("global", lldb.eValueTypeVariableGlobal)
        local = frame0.FindValue("local", lldb.eValueTypeVariableLocal)
        error = lldb.SBError()

        wp_opts = lldb.SBWatchpointOptions()
        wp_opts.SetWatchpointTypeRead(True)
        watchpoint = target.WatchpointCreateByAddress(
            value.GetLoadAddress(), 1, wp_opts, error
        )
        self.assertTrue(
            value and local and watchpoint,
            "Successfully found the values and set a watchpoint",
        )
        self.DebugSBValue(value)
        self.DebugSBValue(local)

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        print(watchpoint)

        # Continue.  Expect the program to stop due to the variable being
        # read, but *not* written to.
        process.Continue()

        if self.TraceOn():
            lldbutil.print_stacktraces(process)

        self.assertTrue(
            local.GetValueAsSigned() > 0, "The local variable has been incremented"
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @expectedFailureWindows
    def test(self):
        self.build()
        (_, process, _, _) = lldbutil.run_to_source_breakpoint(
            self, "main", lldb.SBFileSpec("main.cpp")
        )

        self.expect(
            "expression some_template_func<int, long>(5)",
            error=True,
            substrs=[
                "does not name a template but is followed by template arguments",
                "note: Naming template instantiation not yet supported.",
                "Template functions can be invoked via their mangled name.",
            ],
        )

        self.expect(
            "expression some_template_func<int, long>(5) + some_template_func<int, long>(5)",
            error=True,
            substrs=[
                "does not name a template but is followed by template arguments",
                "does not name a template but is followed by template arguments",
            ],
        )

        self.assertEqual(
            self.res.GetError().count(
                "note: Naming template instantiation not yet supported"
            ),
            1,
        )

        self.expect(
            "expression Foo<int>::smethod()",
            error=True,
            substrs=[
                "no template named 'Foo'",
                "note: Naming template instantiation not yet supported.",
                "Template functions can be invoked via their mangled name.",
            ],
        )



# RUN: %PYTHON -m mlir.dialects.linalg.opdsl.dump_oplib .ops.core_named_ops | FileCheck %s

# Just verify that at least one known op is generated.
# CHECK: name: copy

# verify some special cases: negf->NegFOp, powf->PowFOp
# CHECK cpp_class_name: NegFOp
# CHECK cpp_class_name: PowFOp



# RUN: env SUPPORT_LIB=%mlir_c_runner_utils \
# RUN:   %PYTHON %s | FileCheck %s

import ctypes
import numpy as np
import os
import sys

from mlir import ir
from mlir import runtime as rt

from mlir.dialects import sparse_tensor as st
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects.linalg.opdsl import lang as dsl

_SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(_SCRIPT_PATH)
from tools import sparsifier


@dsl.linalg_structured_op
def matmul_dsl(
    A=dsl.TensorDef(dsl.T, dsl.S.M, dsl.S.K),
    B=dsl.TensorDef(dsl.T, dsl.S.K, dsl.S.N),
    C=dsl.TensorDef(dsl.T, dsl.S.M, dsl.S.N, output=True),
):
    C[dsl.D.m, dsl.D.n] += A[dsl.D.m, dsl.D.k] * B[dsl.D.k, dsl.D.n]


def build_SpMM(attr: st.EncodingAttr):
    """Build SpMM kernel.

    This method generates a linalg op with for matrix multiplication using
    just the Python API. Effectively, a generic linalg op is constructed
    that computes C(i,j) += A(i,k) * B(k,j) for annotated matrix A.
    """
    module = ir.Module.create()
    f64 = ir.F64Type.get()
    a = ir.RankedTensorType.get([3, 4], f64, attr)
    b = ir.RankedTensorType.get([4, 2], f64)
    c = ir.RankedTensorType.get([3, 2], f64)
    arguments = [a, b, c]
    with ir.InsertionPoint(module.body):

        @func.FuncOp.from_py_func(*arguments)
        def spMxM(*args):
            return matmul_dsl(args[0], args[1], outs=[args[2]])

    return module


def boilerplate(attr: st.EncodingAttr):
    """Returns boilerplate main method.

    This method sets up a boilerplate main method that takes three tensors
    (a, b, c), converts the first tensor a into s sparse tensor, and then
    calls the sparse kernel for matrix multiplication. For convenience,
    this part is purely done as string input.
    """
    return f"""
func.func @main(%ad: tensor<3x4xf64>, %b: tensor<4x2xf64>, %c: tensor<3x2xf64>) -> tensor<3x2xf64>
  attributes {{ llvm.emit_c_interface }} {{
  %a = sparse_tensor.convert %ad : tensor<3x4xf64> to tensor<3x4xf64, {attr}>
  %0 = call @spMxM(%a, %b, %c) : (tensor<3x4xf64, {attr}>,
                                  tensor<4x2xf64>,
                                  tensor<3x2xf64>) -> tensor<3x2xf64>
  return %0 : tensor<3x2xf64>
}}
"""


def build_compile_and_run_SpMM(attr: st.EncodingAttr, compiler):
    # Build.
    module = build_SpMM(attr)
    func = str(module.operation.regions[0].blocks[0].operations[0].operation)
    module = ir.Module.parse(func + boilerplate(attr))

    # Compile.
    engine = compiler.compile_and_jit(module)

    # Set up numpy input and buffer for output.
    a = np.array(
        [[1.1, 0.0, 0.0, 1.4], [0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 3.3, 0.0]], np.float64
    )
    b = np.array([[1.0, 2.0], [4.0, 3.0], [5.0, 6.0], [8.0, 7.0]], np.float64)
    c = np.zeros((3, 2), np.float64)

    mem_a = ctypes.pointer(ctypes.pointer(rt.get_ranked_memref_descriptor(a)))
    mem_b = ctypes.pointer(ctypes.pointer(rt.get_ranked_memref_descriptor(b)))
    mem_c = ctypes.pointer(ctypes.pointer(rt.get_ranked_memref_descriptor(c)))
    # Allocate a MemRefDescriptor to receive the output tensor.
    # The buffer itself is allocated inside the MLIR code generation.
    ref_out = rt.make_nd_memref_descriptor(2, ctypes.c_double)()
    mem_out = ctypes.pointer(ctypes.pointer(ref_out))

    # Invoke the kernel and get numpy output.
    # Built-in bufferization uses in-out buffers.
    engine.invoke("main", mem_out, mem_a, mem_b, mem_c)

    # Sanity check on computed result.
    expected = np.matmul(a, b)
    c = rt.ranked_memref_to_numpy(mem_out[0])
    if np.allclose(c, expected):
        pass
    else:
        quit(f"FAILURE")


def main():
    support_lib = os.getenv("SUPPORT_LIB")
    assert support_lib is not None, "SUPPORT_LIB is undefined"
    if not os.path.exists(support_lib):
        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), support_lib)

    # CHECK-LABEL: TEST: testSpMM
    print("\nTEST: testSpMM")
    count = 0
    with ir.Context() as ctx, ir.Location.unknown():
        # Loop over various ways to compile and annotate the SpMM kernel with
        # a *single* sparse tensor. Note that we deliberate do not exhaustively
        # search the full state space to reduce runtime of the test. It is
        # straightforward to adapt the code below to explore more combinations.
        # For these simple orderings, dim2lvl and lvl2dim are the same.
        vl = 1
        e = False
        opt = f"parallelization-strategy=none"
        builder = st.EncodingAttr.build_level_type
        fmt = st.LevelFormat
        prop = st.LevelProperty
        levels = [
            [builder(fmt.compressed, [prop.non_unique]), builder(fmt.singleton)],
            [builder(fmt.dense), builder(fmt.dense)],
            [builder(fmt.dense), builder(fmt.compressed)],
            [builder(fmt.compressed), builder(fmt.dense)],
            [builder(fmt.compressed), builder(fmt.compressed)],
        ]
        orderings = [
            ir.AffineMap.get_permutation([0, 1]),
            ir.AffineMap.get_permutation([1, 0]),
        ]
        bitwidths = [0]
        compiler = sparsifier.Sparsifier(
            extras="", options=opt, opt_level=0, shared_libs=[support_lib]
        )
        for level in levels:
            for ordering in orderings:
                for pwidth in bitwidths:
                    for iwidth in bitwidths:
                        attr = st.EncodingAttr.get(
                            level, ordering, ordering, pwidth, iwidth
                        )
                        build_compile_and_run_SpMM(attr, compiler)
                        count = count + 1
        # CHECK: Passed 10 tests
        print("Passed ", count, "tests")


if __name__ == "__main__":
    main()



"""
Test DIL pointer arithmetic.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestFrameVarDILExprPointerArithmetic(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_pointer_arithmetic(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.experimental.use-DIL true")

        self.expect_var_path("+array", type="int *")
        self.expect_var_path("+array_ref", type="int *")
        self.expect_var_path("+p_int0", type="int *")

        # Binary operations
        self.expect_var_path("p_char", type="const char *")
        self.expect_var_path("p_char + 1", type="const char *")
        self.expect_var_path("p_char + offset", type="const char *")
        self.expect_var_path("p_char5 + -1", type="const char *")
        self.expect_var_path("p_char5 - 1", type="const char *")
        self.expect_var_path("p_char5 - offset", type="const char *")

        self.expect_var_path("my_p_char", type="my_char_ptr")
        self.expect_var_path("my_p_char + 1", type="my_char_ptr")
        self.expect_var_path("my_p_char - 1", type="my_char_ptr")

        self.expect_var_path("*(p_char + 0)", value="'h'")
        self.expect_var_path("*(5 + p_char)", value="'!'")
        self.expect_var_path("*(p_char5 + -5)", value="'h'")
        self.expect_var_path("*(p_char5 - 5)", value="'h'")
        self.expect_var_path("*(p_char - -5)", value="'!'")
        self.expect_var_path("*(p_char5 - offset + 5)", value="'!'")
        self.expect_var_path("*((p_char + offset) - 5)", value="'h'")
        self.expect_var_path("*(p_char + (offset - 5))", value="'h'")

        self.expect_var_path("*p_int0", value="0")
        self.expect_var_path("*cp_int5", value="5")
        self.expect_var_path("*(&*(cp_int5 + 1) - 1)", value="5")

        self.expect_var_path("p_int0 - p_int0", value="0", type="__ptrdiff_t")
        self.expect_var_path("cp_int5 - p_int0", value="5", type="__ptrdiff_t")
        self.expect_var_path("cp_int5 - td_int_ptr0", value="5", type="__ptrdiff_t")
        self.expect_var_path("td_int_ptr0 - cp_int5", value="-5", type="__ptrdiff_t")

        # Check arrays
        self.expect_var_path("array + 1", type="int *")
        self.expect_var_path("1 + array", type="int *")
        self.expect_var_path("array_ref + 1", type="int *")
        self.expect_var_path("1 + array_ref", type="int *")
        self.expect_var_path("array - 1", type="int *")
        self.expect_var_path("array_ref - 1", type="int *")
        self.expect_var_path("array - array", value="0", type="__ptrdiff_t")
        self.expect_var_path("array - array_ref", value="0", type="__ptrdiff_t")
        self.expect_var_path("array_ref - array_ref", value="0", type="__ptrdiff_t")

        # Errors
        self.expect(
            "frame var -- '-p_int0'",
            error=True,
            substrs=["invalid argument type 'int *' to unary expression"],
        )
        self.expect(
            "frame var -- 'cp_int5 - p_char'",
            error=True,
            substrs=[
                "'const int *' and 'const char *' are not pointers to compatible types"
            ],
        )
        self.expect(
            "frame var -- 'p_int0 + cp_int5'",
            error=True,
            substrs=[
                "invalid operands to binary expression ('int *' and 'const int *')"
            ],
        )
        self.expect(
            "frame var -- 'p_void + 1'",
            error=True,
            substrs=["arithmetic on a pointer to void"],
        )
        self.expect(
            "frame var -- 'p_void - 1'",
            error=True,
            substrs=["arithmetic on a pointer to void"],
        )
        self.expect(
            "frame var -- 'p_void - p_char'",
            error=True,
            substrs=[
                "'void *' and 'const char *' are not pointers to compatible types"
            ],
        )
        self.expect(
            "frame var -- 'p_void - p_void'",
            error=True,
            substrs=["arithmetic on pointers to void"],
        )
        self.expect(
            "frame var -- 'pp_void0 - p_char'",
            error=True,
            substrs=[
                "'void **' and 'const char *' are not pointers to compatible types"
            ],
        )
        self.expect(
            "frame var -- 'p_int0 - 1.0'",
            error=True,
            substrs=["invalid operands to binary expression ('int *' and 'double')"],
        )
        self.expect(
            "frame var -- '1.0f + p_int0'",
            error=True,
            substrs=["invalid operands to binary expression ('float' and 'int *')"],
        )
        self.expect(
            "frame var -- '1 - array'",
            error=True,
            substrs=["invalid operands to binary expression ('int' and 'int[10]')"],
        )
        self.expect(
            "frame var -- 'array + array'",
            error=True,
            substrs=["invalid operands to binary expression ('int[10]' and 'int[10]')"],
        )
        self.expect(
            "frame var -- 'array + array'",
            error=True,
            substrs=["invalid operands to binary expression ('int[10]' and 'int[10]')"],
        )
        self.expect(
            "frame var -- 'int_null + 1'",
            error=True,
            substrs=["arithmetic on a nullptr is undefined"],
        )
        self.expect(
            "frame var -- 'int_null - 1'",
            error=True,
            substrs=["arithmetic on a nullptr is undefined"],
        )
        self.expect(
            "frame var -- 'p_char + *((int*) 0)'",
            error=True,
            substrs=["could not get the offset: parent is NULL"],
        )
        self.expect(
            "frame var -- 'p_char - *((int*) 0)'",
            error=True,
            substrs=["could not get the offset: parent is NULL"],
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


# This test case checks for register number mismatch between lldb and gdb stub.
# LLDB client assigns register numbers to target xml registers in increasing
# order starting with regnum = 0, while gdb-remote may specify different regnum
# which is stored as eRegisterKindProcessPlugin. Remote side will use its
# register number in expedited register list, value_regs and invalidate_regnums.
#
# This test creates a ficticious target xml with non-sequential regnums to test
# that correct registers are accessed in all of above mentioned cases.


class TestRemoteRegNums(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    def test(self):
        class MyResponder(MockGDBServerResponder):
            def haltReason(self):
                return "T02thread:1ff0d;threads:1ff0d;thread-pcs:000000010001bc00;00:00bc010001000000;09:c04825ebfe7f0000;"

            def threadStopInfo(self, threadnum):
                return "T02thread:1ff0d;threads:1ff0d;thread-pcs:000000010001bc00;00:00bc010001000000;09:c04825ebfe7f0000;"

            def writeRegisters(self):
                return "E02"

            def readRegisters(self):
                return "E01"

            rax_regnum2_val = "7882773ce0ffffff"
            rbx_regnum4_val = "1122334455667788"

            def readRegister(self, regnum):
                # lldb will try sending "p0" to see if the p packet is supported,
                # give a bogus value; in theory lldb could use this value in the
                # register context and that would be valid behavior.

                # notably, don't give values for registers 1 & 3 -- lldb should
                # get those from the ? stop packet ("T11") and it is a pref regression
                # if lldb is asking for these register values.
                if regnum == 0:
                    return "5555555555555555"
                if regnum == 2:
                    return self.rax_regnum2_val
                if regnum == 4:
                    return self.rbx_regnum4_val

                return "E03"

            def writeRegister(self, regnum, value_hex):
                if regnum == 2:
                    self.rax_regnum2_val = value_hex
                if regnum == 4:
                    self.rbx_regnum4_val = value_hex

                return "OK"

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <target version="1.0">
                          <architecture>i386:x86-64</architecture>
                          <feature name="org.gnu.gdb.i386.core">
                            <reg name="rip" bitsize="64" regnum="0" type="code_ptr" group="general" altname="pc" generic="pc"/>
                            <reg name="rax" bitsize="64" regnum="2" type="code_ptr" group="general"/>
                            <reg name="rbx" bitsize="64" regnum="4" type="code_ptr" group="general"/>
                            <reg name="eax" bitsize="32" regnum="5" value_regnums="2" invalidate_regnums="2" type="code_ptr" group="general"/>
                            <reg name="ebx" bitsize="32" regnum="7" value_regnums="4" invalidate_regnums="4" type="code_ptr" group="general"/>
                            <reg name="rsi" bitsize="64" regnum="9" type="code_ptr" group="general"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))
        process = self.connect(target)

        thread = process.GetThreadAtIndex(0)
        frame = thread.GetFrameAtIndex(0)
        rax = frame.FindRegister("rax").GetValueAsUnsigned()
        eax = frame.FindRegister("eax").GetValueAsUnsigned()
        rbx = frame.FindRegister("rbx").GetValueAsUnsigned()
        ebx = frame.FindRegister("ebx").GetValueAsUnsigned()
        rsi = frame.FindRegister("rsi").GetValueAsUnsigned()
        pc = frame.GetPC()
        rip = frame.FindRegister("rip").GetValueAsUnsigned()

        if self.TraceOn():
            print(
                "Register values: rax == 0x%x, rbx == 0x%x, rsi == 0x%x, pc == 0x%x, rip == 0x%x"
                % (rax, rbx, rsi, pc, rip)
            )

        self.assertEqual(rax, 0xFFFFFFE03C778278)
        self.assertEqual(rbx, 0x8877665544332211)
        self.assertEqual(eax, 0x3C778278)
        self.assertEqual(ebx, 0x44332211)
        self.assertEqual(rsi, 0x00007FFEEB2548C0)
        self.assertEqual(pc, 0x10001BC00)
        self.assertEqual(rip, 0x10001BC00)

        frame.FindRegister("eax").SetValueFromCString("1")
        frame.FindRegister("ebx").SetValueFromCString("0")
        eax = frame.FindRegister("eax").GetValueAsUnsigned()
        ebx = frame.FindRegister("ebx").GetValueAsUnsigned()
        rax = frame.FindRegister("rax").GetValueAsUnsigned()
        rbx = frame.FindRegister("rbx").GetValueAsUnsigned()

        if self.TraceOn():
            print(
                "Register values: rax == 0x%x, rbx == 0x%x, rsi == 0x%x, pc == 0x%x, rip == 0x%x"
                % (rax, rbx, rsi, pc, rip)
            )

        self.assertEqual(rax, 0xFFFFFFE000000001)
        self.assertEqual(rbx, 0x8877665500000000)
        self.assertEqual(eax, 0x00000001)
        self.assertEqual(ebx, 0x00000000)
        self.assertEqual(rsi, 0x00007FFEEB2548C0)
        self.assertEqual(pc, 0x10001BC00)
        self.assertEqual(rip, 0x10001BC00)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCiVarIMPTestCase(TestBase):
    @skipUnlessDarwin
    @no_debug_info_test
    def test_imp_ivar_type(self):
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_name_breakpoint(self, "main")
        frame = thread.GetFrameAtIndex(0)
        lang_info = frame.GetLanguageSpecificData()
        version = lang_info.GetValueForKey("Objective-C runtime version")
        self.assertEqual(version.GetIntegerValue(), 2)



"""
Test lldb-dap setBreakpoints request
"""

from dap_server import Source
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbdap_testcase
import os


class TestDAP_breakpointEvents(lldbdap_testcase.DAPTestCaseBase):
    @skipIfWindows
    def test_breakpoint_events(self):
        """
        This test follows the following steps.
        - Sets a breakpoint in a shared library from the preCommands.
        - Runs and stops at the entry point of a program.
        - Sets two new breakpoints, one in the main executable and one in the shared library
        - Both breakpoint is set but only the the shared library breakpoint is not verified yet.
        - We will then continue and expect to get a breakpoint event that
            informs us the breakpoint in the shared library has "changed"
            and the correct line number should be supplied.
        - We also verify the breakpoint set via the command interpreter should not
            be have breakpoint events sent back to VS Code as the UI isn't able to
            add new breakpoints to their UI.

        Code has been added that tags breakpoints set from VS Code
        DAP packets so we know the IDE knows about them. If VS Code is ever
        able to register breakpoints that aren't initially set in the GUI,
        then we will need to revise this.
        """
        main_source_path = self.getSourcePath("main.cpp")
        main_bp_line = line_number(main_source_path, "main breakpoint 1")
        main_source = Source.build(path=main_source_path)
        program = self.getBuildArtifact("a.out")

        # Set a breakpoint after creating the target by running a command line
        # command. It will eventually resolve and cause a breakpoint changed
        # event to be sent to lldb-dap. We want to make sure we don't send a
        # breakpoint any breakpoints that were set from the command line.
        # Breakpoints that are set via the VS code DAP packets will be
        # registered and marked with a special keyword to ensure we deliver
        # breakpoint events for these breakpoints but not for ones that are not
        # set via the command interpreter.

        shlib_env_key = self.platformContext.shlib_environment_var
        path_separator = self.platformContext.shlib_path_separator
        shlib_env_value = os.getenv(shlib_env_key)
        shlib_env_new_value = (
            self.getBuildDir()
            if shlib_env_value is None
            else (shlib_env_value + path_separator + self.getBuildDir())
        )

        # Set preCommand breakpoint
        func_unique_function_name = "unique_function_name"
        bp_command = f"breakpoint set --name {func_unique_function_name}"
        launch_seq = self.build_and_launch(
            program,
            preRunCommands=[bp_command],
            env={shlib_env_key: shlib_env_new_value},
        )
        self.dap_server.wait_for_event(["initialized"])
        dap_breakpoint_ids = []

        # We set the breakpoints after initialized event.
        # Set and verify new line breakpoint.
        response = self.dap_server.request_setBreakpoints(main_source, [main_bp_line])
        self.assertTrue(response["success"])
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(len(breakpoints), 1, "expects only one line breakpoint")
        main_breakpoint = breakpoints[0]
        main_bp_id = main_breakpoint["id"]
        dap_breakpoint_ids.append(main_bp_id)
        self.assertTrue(
            main_breakpoint["verified"], "expects main breakpoint to be verified"
        )

        # Set and verify new function breakpoint.
        func_foo = "foo"
        response = self.dap_server.request_setFunctionBreakpoints([func_foo])
        self.assertTrue(response["success"])
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(len(breakpoints), 1, "expects only one function breakpoint")
        func_foo_breakpoint = breakpoints[0]
        foo_bp_id = func_foo_breakpoint["id"]
        dap_breakpoint_ids.append(foo_bp_id)
        self.assertFalse(
            func_foo_breakpoint["verified"],
            "expects unique function breakpoint to not be verified",
        )

        self.dap_server.request_configurationDone()
        launch_response = self.dap_server.receive_response(launch_seq)
        self.assertIsNotNone(launch_response)
        self.assertTrue(launch_response["success"])

        # Wait for the next stop (breakpoint foo).
        self.verify_breakpoint_hit([foo_bp_id])
        unique_bp_id = 1

        # Check the breakpoints set in dap is verified.
        verified_breakpoint_ids = []
        events = self.dap_server.wait_for_breakpoint_events()
        for breakpoint_event in events:
            breakpoint_event_body = breakpoint_event["body"]
            if breakpoint_event_body["reason"] != "changed":
                continue
            breakpoint = breakpoint_event_body["breakpoint"]

            if "verified" in breakpoint_event_body:
                self.assertFalse(
                    breakpoint_event_body["verified"],
                    f"expects changed breakpoint to be verified. event: {breakpoint_event}",
                )
            id = breakpoint["id"]
            verified_breakpoint_ids.append(id)

        self.assertIn(main_bp_id, verified_breakpoint_ids)
        self.assertIn(foo_bp_id, verified_breakpoint_ids)
        self.assertNotIn(unique_bp_id, verified_breakpoint_ids)

        # Continue to the unique function breakpoint set from preRunCommands.
        unique_function_stop_event = self.continue_to_next_stop()[0]
        unique_body = unique_function_stop_event["body"]
        self.assertEqual(unique_body["reason"], "breakpoint")
        self.assertIn(unique_bp_id, unique_body["hitBreakpointIds"])

        # Clear line and function breakpoints and exit.
        self.dap_server.request_setFunctionBreakpoints([])
        self.dap_server.request_setBreakpoints(main_source, [])
        self.continue_to_exit()



"""
Test calling a function, stopping in the call, continue and gather the result on stop.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprCommandCallStopContinueTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.c.

    def test(self):
        """Test gathering result from interrupted function call."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.cpp",
            line_number("main.cpp", '{5, "five"}'),
            num_expected_locations=-1,
            loc_exact=True,
        )

        self.expect(
            "expr -i false -- returnsFive()",
            error=True,
            substrs=["Expression execution hit a breakpoint: breakpoint"],
        )

        self.runCmd("continue", "Continue completed")
        self.expect(
            "thread list",
            substrs=[
                "stop reason = User Expression thread plan",
                r'Completed expression: (Five) $0 = (number = 5, name = "five")',
            ],
        )



"""
Test lldb data formatter subsystem.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StdValarrayDataFormatterTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def do_test(self):
        (self.target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp", False)
        )

        #
        # std::valarray
        #

        self.expect(
            "frame variable va_int",
            substrs=[
                "va_int = size=4",
                "[0] = 0",
                "[1] = 0",
                "[2] = 0",
                "[3] = 0",
                "}",
            ],
        )

        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect(
            "frame variable va_int",
            substrs=[
                "va_int = size=4",
                "[0] = 1",
                "[1] = 12",
                "[2] = 123",
                "[3] = 1234",
                "}",
            ],
        )

        # check access-by-index
        self.expect("frame variable va_int[0]", substrs=["1"])
        self.expect("frame variable va_int[1]", substrs=["12"])
        self.expect("frame variable va_int[2]", substrs=["123"])
        self.expect("frame variable va_int[3]", substrs=["1234"])
        self.expect(
            "frame variable va_int[4]",
            error=True,
            substrs=['array index 4 is not valid for "(valarray'],
        )

        self.expect(
            "frame variable va_double",
            substrs=[
                "va_double = size=4",
                "[0] = 1",
                "[1] = 0.5",
                "[2] = 0.25",
                "[3] = 0.125",
                "}",
            ],
        )

        # check access-by-index
        self.expect("frame variable va_double[0]", substrs=["1"])
        self.expect("frame variable va_double[1]", substrs=["0.5"])
        self.expect("frame variable va_double[2]", substrs=["0.25"])
        self.expect("frame variable va_double[3]", substrs=["0.125"])
        self.expect(
            "frame variable va_double[4]",
            error=True,
            substrs=['array index 4 is not valid for "(valarray'],
        )

        #
        # std::slice_array
        #

        self.expect(
            "frame variable sa",
            substrs=[
                "sa = stride=2 size=4",
                "[0] = 11",
                "[1] = 13",
                "[2] = 15",
                "[3] = 17",
                "}",
            ],
        )

        # check access-by-index
        self.expect("frame variable sa[0]", substrs=["11"])
        self.expect("frame variable sa[1]", substrs=["13"])
        self.expect("frame variable sa[2]", substrs=["15"])
        self.expect("frame variable sa[3]", substrs=["17"])
        self.expect(
            "frame variable sa[4]",
            error=True,
            substrs=['array index 4 is not valid for "(slice_array'],
        )

        #
        # std::gslice_array
        #

        self.expect(
            "frame variable ga",
            substrs=[
                "ga = size=3",
                "[0] -> [3] = 13",
                "[1] -> [4] = 14",
                "[2] -> [5] = 15",
                "}",
            ],
        )

        # check access-by-index
        self.expect("frame variable ga[0]", substrs=["13"])
        self.expect("frame variable ga[1]", substrs=["14"])
        self.expect("frame variable ga[2]", substrs=["15"])
        self.expect(
            "frame variable ga[3]",
            error=True,
            substrs=['array index 3 is not valid for "(gslice_array'],
        )
        #
        # std::mask_array
        #

        self.expect(
            "frame variable ma",
            substrs=[
                "ma = size=2",
                "[0] -> [1] = 11",
                "[1] -> [2] = 12",
                "}",
            ],
        )

        # check access-by-index
        self.expect("frame variable ma[0]", substrs=["11"])
        self.expect("frame variable ma[1]", substrs=["12"])
        self.expect(
            "frame variable ma[2]",
            error=True,
            substrs=['array index 2 is not valid for "(mask_array'],
        )

        #
        # std::indirect_array
        #

        self.expect(
            "frame variable ia",
            substrs=[
                "ia = size=3",
                "[0] -> [3] = 13",
                "[1] -> [6] = 16",
                "[2] -> [9] = 19",
                "}",
            ],
        )

        # check access-by-index
        self.expect("frame variable ia[0]", substrs=["13"])
        self.expect("frame variable ia[1]", substrs=["16"])
        self.expect("frame variable ia[2]", substrs=["19"])
        self.expect(
            "frame variable ia[3]",
            error=True,
            substrs=['array index 3 is not valid for "(indirect_array'],
        )

    @add_test_categories(["libc++"])
    def test_libcxx(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test()



# coding=utf8
"""
Test that C++ supports wchar_t correctly.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class CxxWCharTTestCase(TestBase):
    def test(self):
        """Test that C++ supports wchar_t correctly."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        # Check that we correctly report templates on wchar_t
        self.expect_var_path("foo_y", type="Foo<wchar_t>")

        # Check that we correctly report templates on int
        self.expect_var_path("foo_x", type="Foo<int>")

        # Check that we correctly report wchar_t
        self.expect_var_path("foo_y.object", type="wchar_t")

        # Check that we correctly report int
        self.expect_var_path("foo_x.object", type="int")

        # Check that we can run expressions that return wchar_t
        self.expect("expression L'a'", substrs=["(wchar_t) $", "L'a'"])

        # Mazel Tov if this works!
        self.expect(
            "frame variable mazeltov",
            substrs=["(const wchar_t *) mazeltov = ", 'L"מזל טוב"'],
        )

        self.expect("frame variable ws_NULL", substrs=["(wchar_t *) ws_NULL = 0x0"])
        self.expect("frame variable ws_empty", substrs=[' L""'])

        self.expect(
            "frame variable array", substrs=["L\"Hey, I'm a super wchar_t string"]
        )
        self.expect("frame variable array", substrs=["[0]"], matching=False)

        self.expect("frame variable wchar_zero", substrs=["L'\\0'"])
        self.expect("expression wchar_zero", substrs=["L'\\0'"])



import lldb


@lldb.command()
def decorated1(debugger, args, exe_ctx, result, dict):
    """
    Python command defined by @lldb.command
    """
    print("hello from decorated1", file=result)


@lldb.command(doc="Python command defined by @lldb.command")
def decorated2(debugger, args, exe_ctx, result, dict):
    """
    This docstring is overridden.
    """
    print("hello from decorated2", file=result)


@lldb.command()
def decorated3(debugger, args, result, dict):
    """
    Python command defined by @lldb.command
    """
    print("hello from decorated3", file=result)


@lldb.command("decorated4")
def _decorated4(debugger, args, exe_ctx, result, dict):
    """
    Python command defined by @lldb.command
    """
    print("hello from decorated4", file=result)



"""
Test stopping at a breakpoint in an expression, and unwinding from there.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class UnwindFromExpressionTest(TestBase):
    main_spec = lldb.SBFileSpec("main.cpp", False)

    def build_and_run_to_bkpt(self):
        self.build()

        (target, process, self.thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// Set a breakpoint here to get started", self.main_spec
        )

        # Next set a breakpoint in this function, set up Expression options to stop on
        # breakpoint hits, and call the function.
        self.fun_bkpt = self.target().BreakpointCreateBySourceRegex(
            "// Stop inside the function here.", self.main_spec
        )
        self.assertTrue(self.fun_bkpt, VALID_BREAKPOINT)

    @no_debug_info_test
    @expectedFailureAll(bugnumber="llvm.org/pr33164")
    def test_conditional_bktp(self):
        """
        Test conditional breakpoint handling in the IgnoreBreakpoints = False case
        """
        self.build_and_run_to_bkpt()

        self.fun_bkpt.SetCondition("0")  # Should not get hit
        options = lldb.SBExpressionOptions()
        options.SetIgnoreBreakpoints(False)
        options.SetUnwindOnError(False)

        main_frame = self.thread.GetFrameAtIndex(0)
        val = main_frame.EvaluateExpression("second_function(47)", options)
        self.assertSuccess(val.GetError(), "We did complete the execution.")
        self.assertEqual(47, val.GetValueAsSigned())

    @add_test_categories(["pyapi"])
    @expectedFlakeyNetBSD
    def test_unwind_expression(self):
        """Test unwinding from an expression."""
        self.build_and_run_to_bkpt()

        # Run test with varying one thread timeouts to also test the halting
        # logic in the IgnoreBreakpoints = False case
        self.do_unwind_test(self.thread, self.fun_bkpt, 1000)
        self.do_unwind_test(self.thread, self.fun_bkpt, 100000)

    def do_unwind_test(self, thread, bkpt, timeout):
        #
        # Use Python API to evaluate expressions while stopped in a stack frame.
        #
        main_frame = thread.GetFrameAtIndex(0)

        options = lldb.SBExpressionOptions()
        options.SetIgnoreBreakpoints(False)
        options.SetUnwindOnError(False)
        options.SetOneThreadTimeoutInMicroSeconds(timeout)

        val = main_frame.EvaluateExpression("a_function_to_call()", options)

        self.assertTrue(val.GetError().Fail(), "We did not complete the execution.")
        error_str = val.GetError().GetCString()
        self.assertIn(
            "Expression execution hit a breakpoint: breakpoint",
            error_str,
            "And the reason was right.",
        )

        thread = lldbutil.get_one_thread_stopped_at_breakpoint(self.process(), bkpt)
        self.assertTrue(thread.IsValid(), "We are indeed stopped at our breakpoint")

        # Now unwind the expression, and make sure we got back to where we
        # started.
        self.assertSuccess(
            thread.UnwindInnermostExpression(), "We succeeded in unwinding"
        )

        cur_frame = thread.GetFrameAtIndex(0)
        self.assertTrue(cur_frame.IsEqual(main_frame), "We got back to the main frame.")



"""
Test some ARM instruction emulation.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ARMEmulationTestCase(TestBase):
    @no_debug_info_test
    def test_thumb_emulations(self):
        test_dir = os.path.join(self.getSourceDir(), "new-test-files")
        files = os.listdir(test_dir)
        thumb_files = list()
        for f in files:
            if f.endswith("-thumb.dat"):
                thumb_files.append(f)

        for f in thumb_files:
            test_file = os.path.join(test_dir, f)
            self.run_a_single_test(test_file)

    @no_debug_info_test
    def test_arm_emulations(self):
        test_dir = os.path.join(self.getSourceDir(), "new-test-files")
        files = os.listdir(test_dir)
        arm_files = list()
        for f in files:
            if f.endswith("-arm.dat"):
                arm_files.append(f)

        for f in arm_files:
            test_file = os.path.join(test_dir, f)
            self.run_a_single_test(test_file)

    def run_a_single_test(self, filename):
        insn = lldb.SBInstruction()
        stream = lldb.SBStream()
        success = insn.TestEmulation(stream, filename)
        output = stream.GetData()
        if self.TraceOn():
            print("\nRunning test " + os.path.basename(filename))
            print(output)

        self.assertTrue(success, "Emulation test {} failed.".format(filename))



"""
This is a sanity check that verifies that the module cache path is set
correctly and points inside the default test build directory.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class ModuleCacheSanityTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        self.expect(
            "settings show symbols.clang-modules-cache-path",
            substrs=["lldb-test-build.noindex", "module-cache-lldb"],
        )



"""
Make sure that if threads are suspended outside of lldb, debugserver
won't make them run, even if we call an expression on the thread.
"""

import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestSuspendedThreadHandling(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    def test_suspended_threads(self):
        """Test that debugserver doesn't disturb the suspend count of a thread
        that has been suspended from within a program, when navigating breakpoints
        on other threads, or calling functions both on the suspended thread and
        on other threads."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.suspended_thread_test()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Set up your test case here. If your test doesn't need any set up then
        # remove this method from your TestCase class.

    def try_an_expression(self, thread, correct_value, test_bp):
        frame = thread.frames[0]

        value = frame.EvaluateExpression("function_to_call()")
        self.assertSuccess(value.GetError(), "Successfully called the function")
        self.assertEqual(
            value.GetValueAsSigned(), correct_value, "Got expected value for expression"
        )

        # Again, make sure we didn't let the suspend thread breakpoint run:
        self.assertEqual(
            test_bp.GetHitCount(),
            0,
            "First expression allowed the suspend thread to run",
        )

    def make_bkpt(self, pattern):
        bp = self.target.BreakpointCreateBySourceRegex(pattern, self.main_source_file)
        self.assertEqual(bp.GetNumLocations(), 1, "Locations for %s" % (pattern))
        return bp

    def suspended_thread_test(self):
        (self.target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Stop here to get things going", self.main_source_file
        )

        # Make in the running thread, so the we will have to stop a number of times
        # while handling breakpoints.  The first couple of times we hit it we will
        # run expressions as well.  Make sure we don't let the suspended thread run
        # during those operations.
        rt_bp = self.make_bkpt("Break here to show we can handle breakpoints")

        # Make a breakpoint that we will hit when the running thread exits:
        rt_exit_bp = self.make_bkpt("Break here after thread_join")

        # Make a breakpoint in the suspended thread.  We should not hit this till we
        # resume it after joining the running thread.
        st_bp = self.make_bkpt("We allowed the suspend thread to run")

        # Make a breakpoint after pthread_join of the suspend thread to ensure
        # that we didn't keep the thread from exiting normally
        st_exit_bp = self.make_bkpt(
            " Break here to make sure the thread exited normally"
        )

        threads = lldbutil.continue_to_breakpoint(process, rt_bp)
        self.assertEqual(len(threads), 1, "Hit the running_func breakpoint")

        # Make sure we didn't hit the suspend thread breakpoint:
        self.assertEqual(
            st_bp.GetHitCount(), 0, "Continue allowed the suspend thread to run"
        )

        # Now try an expression on the running thread:
        self.try_an_expression(threads[0], 0, st_bp)

        # Continue, and check the same things:
        threads = lldbutil.continue_to_breakpoint(process, rt_bp)
        self.assertEqual(len(threads), 1, "We didn't hit running breakpoint")

        # Try an expression on the suspended thread:
        thread = lldb.SBThread()
        for thread in process.threads:
            th_name = thread.GetName()
            if th_name is None:
                continue
            if "Look for me" in th_name:
                break
        self.assertTrue(thread.IsValid(), "We found the suspend thread.")
        self.try_an_expression(thread, 1, st_bp)

        # Now set the running thread breakpoint to auto-continue and let it
        # run a bit to make sure we still don't let the suspend thread run.
        rt_bp.SetAutoContinue(True)
        threads = lldbutil.continue_to_breakpoint(process, rt_exit_bp)
        self.assertEqual(len(threads), 1)
        self.assertEqual(
            st_bp.GetHitCount(), 0, "Continue again let suspended thread run"
        )

        # Now continue and we SHOULD hit the suspend_func breakpoint:
        threads = lldbutil.continue_to_breakpoint(process, st_bp)
        self.assertEqual(len(threads), 1, "The thread resumed successfully")

        # Finally, continue again and we should get out of the last pthread_join
        # and the process should be about to exit
        threads = lldbutil.continue_to_breakpoint(process, st_exit_bp)
        self.assertEqual(len(threads), 1, "pthread_join exited successfully")



import lldb


class MyStringSynthProvider:
    def __init__(self, valobj, dict):
        self.valobj = valobj

    def num_children(self, max_num_children):
        return 0

    def has_children(self):
        return False



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @skipUnlessDarwin
    def test_synthetic(self):
        self.build()
        if self.TraceOn():
            self.expect("log enable -v lldb formatters")

        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp")
        )

        frame = thread.selected_frame
        account = frame.var("acc")
        self.assertEqual(account.num_children, 1)
        self.assertEqual(account.child[0].name, "username")

        self.expect("v acc", matching=False, substrs=["password"])

    @skipUnlessDarwin
    def test_update_reuse(self):
        self.build()

        log = self.getBuildArtifact("formatter.log")
        self.runCmd(f"log enable lldb formatters -f {log}")

        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp")
        )

        frame = thread.selected_frame
        account = frame.var("acc")
        self.assertEqual(account.num_children, 1)

        self.filecheck(f"platform shell cat {log}", __file__)
        # CHECK: Bytecode formatter can reuse @update: true (type: `Account`, name: `acc`)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.m"))

        log = self.getBuildArtifact("expr.log")
        self.runCmd(f"log enable lldb expr -f {log}")

        self.expect(
            "expr -O -- bad",
            substrs=["`po` was unsuccessful, running `p` instead\n", "(Bad *) 0x"],
        )
        self.filecheck_log(log, __file__, f"-check-prefix=CHECK-EXPR")
        # CHECK-EXPR: Object description fallback due to error: could not evaluate print object function: expression interrupted

        self.expect(
            "dwim-print -O -- bad",
            substrs=["`po` was unsuccessful, running `p` instead\n", "_lookHere = NO"],
        )
        self.filecheck_log(log, __file__, f"-check-prefix=CHECK-DWIM-PRINT")
        # CHECK-DWIM-PRINT: Object description fallback due to error: could not evaluate print object function: expression interrupted



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetStartAddress()
    obj.GetEndAddress()
    obj.GetFileSpec()
    obj.GetLine()
    obj.GetColumn()
    obj.GetDescription(lldb.SBStream())
